from __future__ import annotations
from typing import (Any, List)
from .reflection import (TypeInfo, class_type)
from .string import (join, is_null_or_empty, format, substring)
from .types import to_string
from .util import (int32_to_string, clear)

def expr_1() -> TypeInfo:
    return class_type("System.Text.StringBuilder", None, StringBuilder)


class StringBuilder:
    def __init__(self, value: str, capacity: int) -> None:
        self.buf = []
        if not is_null_or_empty(value):
            (self.buf.append(value))
        
    
    def __str__(self) -> str:
        __ : StringBuilder = self
        return join("", __.buf)
    

StringBuilder_reflection = expr_1

def StringBuilder__ctor_Z18115A39(value: str, capacity: int) -> StringBuilder:
    return StringBuilder(value, capacity)


def StringBuilder__ctor_Z524259A4(capacity: int) -> StringBuilder:
    return StringBuilder__ctor_Z18115A39("", capacity)


def StringBuilder__ctor_Z721C83C5(value: str) -> StringBuilder:
    return StringBuilder__ctor_Z18115A39(value, 16)


def StringBuilder__ctor() -> StringBuilder:
    return StringBuilder__ctor_Z18115A39("", 16)


def StringBuilder__Append_Z721C83C5(x: StringBuilder, s: str) -> StringBuilder:
    (x.buf.append(s))
    return x


def StringBuilder__Append_244C7CD6(x: StringBuilder, c: str) -> StringBuilder:
    (x.buf.append(c))
    return x


def StringBuilder__Append_Z524259A4(x: StringBuilder, o: int) -> StringBuilder:
    (x.buf.append(int32_to_string(o)))
    return x


def StringBuilder__Append_5E38073B(x: StringBuilder, o: float) -> StringBuilder:
    (x.buf.append(to_string(o)))
    return x


def StringBuilder__Append_Z1FBCCD16(x: StringBuilder, o: bool) -> StringBuilder:
    (x.buf.append(to_string(o)))
    return x


def StringBuilder__Append_4E60E31B(x: StringBuilder, o: Any=None) -> StringBuilder:
    (x.buf.append(to_string(o)))
    return x


def StringBuilder__Append_695F1130(x: StringBuilder, cs: List[str]) -> StringBuilder:
    (x.buf.append(''.join(cs)))
    return x


def StringBuilder__Append_43A65C09(x: StringBuilder, s: StringBuilder) -> StringBuilder:
    (x.buf.append(to_string(s)))
    return x


def StringBuilder__AppendFormat_433E080(x: StringBuilder, fmt: str, o: Any=None) -> StringBuilder:
    (x.buf.append(format(fmt, o)))
    return x


def StringBuilder__AppendLine(x: StringBuilder) -> StringBuilder:
    (x.buf.append("\n"))
    return x


def StringBuilder__AppendLine_Z721C83C5(x: StringBuilder, s: str) -> StringBuilder:
    (x.buf.append(s))
    (x.buf.append("\n"))
    return x


def StringBuilder__get_Length(x: StringBuilder) -> int:
    len_1 : int = 0
    for i in range(len(x.buf) - 1, 0 - 1, -1):
        len_1 = (len_1 + len(x.buf[i])) or 0
    return len_1


def StringBuilder__ToString_Z37302880(x: StringBuilder, first_index: int, length: int) -> str:
    return substring(to_string(x), first_index, length)


def StringBuilder__Clear(x: StringBuilder) -> StringBuilder:
    clear(x.buf)
    return x





from __future__ import annotations
from typing import List
from ...fable_library.array import (fold, exists)
from ...fs_hafas_python.context import Context
from ...fs_hafas_python.extra_types import (IndexMap_2, IndexMap_2__set_Item_541DA560, IndexMap_2__ctor_2B594)
from ...fs_hafas_python.types_hafas_client import ProductType

def parse_bitmask(ctx: Context, bitmask: int) -> IndexMap_2[str, bool]:
    array_1 : List[ProductType] = ctx.profile.products
    def folder(m: IndexMap_2[str, bool], p: ProductType, ctx: Context=ctx, bitmask: int=bitmask) -> IndexMap_2[str, bool]:
        def predicate(b: int, m: IndexMap_2[str, bool]=m, p: ProductType=p) -> bool:
            return (b & bitmask) != 0
        
        IndexMap_2__set_Item_541DA560(m, p.id, exists(predicate, p.bitmasks))
        return m
    
    return fold(folder, IndexMap_2__ctor_2B594(False), array_1)





from __future__ import annotations
from typing import (Any, List, Generic, TypeVar, Callable, Optional)
from .option import some
from .reflection import (TypeInfo, union_type)
from .types import Union

_T = TypeVar("_T")

_TERROR = TypeVar("_TERROR")

_A_ = TypeVar("_A_")

_B_ = TypeVar("_B_")

_C_ = TypeVar("_C_")

_T1 = TypeVar("_T1")

_T2 = TypeVar("_T2")

_T3 = TypeVar("_T3")

_T4 = TypeVar("_T4")

_T5 = TypeVar("_T5")

_T6 = TypeVar("_T6")

_T7 = TypeVar("_T7")

def expr_40(gen0: TypeInfo, gen1: TypeInfo) -> TypeInfo:
    return union_type("FSharp.Core.FSharpResult`2", [gen0, gen1], FSharpResult_2, lambda: [[("ResultValue", gen0)], [("ErrorValue", gen1)]])


class FSharpResult_2(Union, Generic[_T, _TERROR]):
    def __init__(self, tag: int, *fields: Any) -> None:
        super().__init__()
        self.tag : int = tag or 0
        self.fields : List[Any] = list(fields)
    
    @staticmethod
    def cases() -> List[str]:
        return ["Ok", "Error"]
    

FSharpResult_2_reflection = expr_40

def Result_Map(mapping: Callable[[_A_], _B_], result: FSharpResult_2[_A_, _C_]) -> FSharpResult_2[_B_, _C_]:
    if result.tag == 0:
        return FSharpResult_2(0, mapping(result.fields[0]))
    
    else: 
        return FSharpResult_2(1, result.fields[0])
    


def Result_MapError(mapping: Callable[[_A_], _B_], result: FSharpResult_2[_C_, _A_]) -> FSharpResult_2[_C_, _B_]:
    if result.tag == 0:
        return FSharpResult_2(0, result.fields[0])
    
    else: 
        return FSharpResult_2(1, mapping(result.fields[0]))
    


def Result_Bind(binder: Callable[[_A_], FSharpResult_2[_B_, _C_]], result: FSharpResult_2[_A_, _C_]) -> FSharpResult_2[_B_, _C_]:
    if result.tag == 0:
        return binder(result.fields[0])
    
    else: 
        return FSharpResult_2(1, result.fields[0])
    


def expr_41(gen0: TypeInfo, gen1: TypeInfo) -> TypeInfo:
    return union_type("FSharp.Core.FSharpChoice`2", [gen0, gen1], FSharpChoice_2, lambda: [[("Item", gen0)], [("Item", gen1)]])


class FSharpChoice_2(Union, Generic[_T1, _T2]):
    def __init__(self, tag: int, *fields: Any) -> None:
        super().__init__()
        self.tag : int = tag or 0
        self.fields : List[Any] = list(fields)
    
    @staticmethod
    def cases() -> List[str]:
        return ["Choice1Of2", "Choice2Of2"]
    

FSharpChoice_2_reflection = expr_41

def expr_42(gen0: TypeInfo, gen1: TypeInfo, gen2: TypeInfo) -> TypeInfo:
    return union_type("FSharp.Core.FSharpChoice`3", [gen0, gen1, gen2], FSharpChoice_3, lambda: [[("Item", gen0)], [("Item", gen1)], [("Item", gen2)]])


class FSharpChoice_3(Union, Generic[_T1, _T2, _T3]):
    def __init__(self, tag: int, *fields: Any) -> None:
        super().__init__()
        self.tag : int = tag or 0
        self.fields : List[Any] = list(fields)
    
    @staticmethod
    def cases() -> List[str]:
        return ["Choice1Of3", "Choice2Of3", "Choice3Of3"]
    

FSharpChoice_3_reflection = expr_42

def expr_43(gen0: TypeInfo, gen1: TypeInfo, gen2: TypeInfo, gen3: TypeInfo) -> TypeInfo:
    return union_type("FSharp.Core.FSharpChoice`4", [gen0, gen1, gen2, gen3], FSharpChoice_4, lambda: [[("Item", gen0)], [("Item", gen1)], [("Item", gen2)], [("Item", gen3)]])


class FSharpChoice_4(Union, Generic[_T1, _T2, _T3, _T4]):
    def __init__(self, tag: int, *fields: Any) -> None:
        super().__init__()
        self.tag : int = tag or 0
        self.fields : List[Any] = list(fields)
    
    @staticmethod
    def cases() -> List[str]:
        return ["Choice1Of4", "Choice2Of4", "Choice3Of4", "Choice4Of4"]
    

FSharpChoice_4_reflection = expr_43

def expr_44(gen0: TypeInfo, gen1: TypeInfo, gen2: TypeInfo, gen3: TypeInfo, gen4: TypeInfo) -> TypeInfo:
    return union_type("FSharp.Core.FSharpChoice`5", [gen0, gen1, gen2, gen3, gen4], FSharpChoice_5, lambda: [[("Item", gen0)], [("Item", gen1)], [("Item", gen2)], [("Item", gen3)], [("Item", gen4)]])


class FSharpChoice_5(Union, Generic[_T1, _T2, _T3, _T4, _T5]):
    def __init__(self, tag: int, *fields: Any) -> None:
        super().__init__()
        self.tag : int = tag or 0
        self.fields : List[Any] = list(fields)
    
    @staticmethod
    def cases() -> List[str]:
        return ["Choice1Of5", "Choice2Of5", "Choice3Of5", "Choice4Of5", "Choice5Of5"]
    

FSharpChoice_5_reflection = expr_44

def expr_45(gen0: TypeInfo, gen1: TypeInfo, gen2: TypeInfo, gen3: TypeInfo, gen4: TypeInfo, gen5: TypeInfo) -> TypeInfo:
    return union_type("FSharp.Core.FSharpChoice`6", [gen0, gen1, gen2, gen3, gen4, gen5], FSharpChoice_6, lambda: [[("Item", gen0)], [("Item", gen1)], [("Item", gen2)], [("Item", gen3)], [("Item", gen4)], [("Item", gen5)]])


class FSharpChoice_6(Union, Generic[_T1, _T2, _T3, _T4, _T5, _T6]):
    def __init__(self, tag: int, *fields: Any) -> None:
        super().__init__()
        self.tag : int = tag or 0
        self.fields : List[Any] = list(fields)
    
    @staticmethod
    def cases() -> List[str]:
        return ["Choice1Of6", "Choice2Of6", "Choice3Of6", "Choice4Of6", "Choice5Of6", "Choice6Of6"]
    

FSharpChoice_6_reflection = expr_45

def expr_46(gen0: TypeInfo, gen1: TypeInfo, gen2: TypeInfo, gen3: TypeInfo, gen4: TypeInfo, gen5: TypeInfo, gen6: TypeInfo) -> TypeInfo:
    return union_type("FSharp.Core.FSharpChoice`7", [gen0, gen1, gen2, gen3, gen4, gen5, gen6], FSharpChoice_7, lambda: [[("Item", gen0)], [("Item", gen1)], [("Item", gen2)], [("Item", gen3)], [("Item", gen4)], [("Item", gen5)], [("Item", gen6)]])


class FSharpChoice_7(Union, Generic[_T1, _T2, _T3, _T4, _T5, _T6, _T7]):
    def __init__(self, tag: int, *fields: Any) -> None:
        super().__init__()
        self.tag : int = tag or 0
        self.fields : List[Any] = list(fields)
    
    @staticmethod
    def cases() -> List[str]:
        return ["Choice1Of7", "Choice2Of7", "Choice3Of7", "Choice4Of7", "Choice5Of7", "Choice6Of7", "Choice7Of7"]
    

FSharpChoice_7_reflection = expr_46

def Choice_makeChoice1Of2(x: Optional[_T1]=None) -> FSharpChoice_2[_T1, _A_]:
    return FSharpChoice_2(0, x)


def Choice_makeChoice2Of2(x: Optional[_T2]=None) -> FSharpChoice_2[_A_, _T2]:
    return FSharpChoice_2(1, x)


def Choice_tryValueIfChoice1Of2(x: FSharpChoice_2[_T1, _T2]) -> Optional[_T1]:
    if x.tag == 0:
        return some(x.fields[0])
    
    else: 
        return None
    


def Choice_tryValueIfChoice2Of2(x: FSharpChoice_2[_T1, _T2]) -> Optional[_T2]:
    if x.tag == 1:
        return some(x.fields[0])
    
    else: 
        return None
    





from __future__ import annotations
from typing import (TypeVar, Any, Optional, List, Tuple, Callable, Generic)
from .array import find_index
from .map_util import (try_get_value, get_item_from_dict)
from .option import some
from .reflection import (TypeInfo, class_type)
from .seq import (concat, iterate_indexed, map, iterate)
from .types import FSharpRef
from .util import (get_enumerator, IEnumerator, to_iterator, ignore, IEnumerable, IEqualityComparer, dispose)

_T = TypeVar("_T")

def expr_28(gen0: TypeInfo) -> TypeInfo:
    return class_type("Fable.Collections.HashSet", [gen0], HashSet)


class HashSet(Generic[_T]):
    def __init__(self, items: IEnumerable[_T], comparer: IEqualityComparer[Any]) -> None:
        this : FSharpRef[HashSet[_T]] = FSharpRef(None)
        self.comparer = comparer
        this.contents = self
        self.hash_map = dict([])
        self.init_00409_002D1 = 1
        with get_enumerator(items) as enumerator:
            while enumerator.System_Collections_IEnumerator_MoveNext():
                item : _T = enumerator.System_Collections_Generic_IEnumerator_00601_get_Current()
                ignore(HashSet__Add_2B595(this.contents, item))
    
    @property
    def Symbol_toStringTag(self) -> str:
        return "HashSet"
    
    def to_json(self, _key: str) -> Any:
        this : HashSet[_T] = self
        return list(this)
    
    def System_Collections_IEnumerable_GetEnumerator(self) -> IEnumerator[Any]:
        this : HashSet[_T] = self
        return get_enumerator(this)
    
    def GetEnumerator(self) -> IEnumerator[_T]:
        this : HashSet[_T] = self
        return get_enumerator(concat(this.hash_map.values()))
    
    def __iter__(self) -> IEnumerator[_T]:
        return to_iterator(self.GetEnumerator())
    
    def System_Collections_Generic_ICollection_00601_Add2B595(self, item: Optional[_T]=None) -> None:
        this : HashSet[_T] = self
        ignore(HashSet__Add_2B595(this, item))
    
    def System_Collections_Generic_ICollection_00601_Clear(self) -> None:
        this : HashSet[_T] = self
        HashSet__Clear(this)
    
    def System_Collections_Generic_ICollection_00601_Contains2B595(self, item: Optional[_T]=None) -> bool:
        this : HashSet[_T] = self
        return HashSet__Contains_2B595(this, item)
    
    def System_Collections_Generic_ICollection_00601_CopyToZ2E171D71(self, array: List[_T], array_index: int) -> None:
        this : HashSet[_T] = self
        def action(i: int, e: _T) -> None:
            array[array_index + i] = e
        
        iterate_indexed(action, this)
    
    def System_Collections_Generic_ICollection_00601_get_Count(self) -> int:
        this : HashSet[_T] = self
        return HashSet__get_Count(this)
    
    def System_Collections_Generic_ICollection_00601_get_IsReadOnly(self) -> bool:
        return False
    
    def System_Collections_Generic_ICollection_00601_Remove2B595(self, item: Optional[_T]=None) -> bool:
        this : HashSet[_T] = self
        return HashSet__Remove_2B595(this, item)
    
    @property
    def size(self) -> int:
        this : HashSet[_T] = self
        return HashSet__get_Count(this)
    
    def add(self, k: Optional[_T]=None) -> Set_1[_T]:
        this : HashSet[_T] = self
        ignore(HashSet__Add_2B595(this, k))
        return this
    
    def clear(self) -> None:
        this : HashSet[_T] = self
        HashSet__Clear(this)
    
    def delete(self, k: Optional[_T]=None) -> bool:
        this : HashSet[_T] = self
        return HashSet__Remove_2B595(this, k)
    
    def has(self, k: Optional[_T]=None) -> bool:
        this : HashSet[_T] = self
        return HashSet__Contains_2B595(this, k)
    
    def keys(self) -> IEnumerable[_T]:
        this : HashSet[_T] = self
        def mapping(x: Optional[_T]=None) -> _T:
            return x
        
        return map(mapping, this)
    
    def values(self) -> IEnumerable[_T]:
        this : HashSet[_T] = self
        def mapping(x: Optional[_T]=None) -> _T:
            return x
        
        return map(mapping, this)
    
    def entries(self) -> IEnumerable[Tuple[_T, _T]]:
        this : HashSet[_T] = self
        def mapping(v: Optional[_T]=None) -> Tuple[_T, _T]:
            return (v, v)
        
        return map(mapping, this)
    
    def for_each(self, f: Callable[[_T, _T, Set_1[_T]], None], this_arg: Optional[Any]=None) -> None:
        this : HashSet[_T] = self
        def action(x: Optional[_T]=None) -> None:
            f(x, x, this)
        
        iterate(action, this)
    

HashSet_reflection = expr_28

def HashSet__ctor_Z6150332D(items: IEnumerable[_T], comparer: IEqualityComparer[Any]) -> HashSet[_T]:
    return HashSet(items, comparer)


def HashSet__TryFindIndex_2B595(this: HashSet[_T], k: _T) -> Tuple[bool, int, int]:
    h : int = this.comparer.GetHashCode(k) or 0
    match_value : Tuple[bool, List[_T]]
    out_arg : List[_T] = None
    def arrow_29(this: HashSet[_T]=this, k: _T=k) -> List[_T]:
        return out_arg
    
    def arrow_30(v: List[_T], this: HashSet[_T]=this, k: _T=k) -> None:
        nonlocal out_arg
        out_arg = v
    
    match_value = (try_get_value(this.hash_map, h, FSharpRef(arrow_29, arrow_30)), out_arg)
    if match_value[0]:
        def arrow_31(v_1: Optional[_T]=None, this: HashSet[_T]=this, k: _T=k) -> bool:
            return this.comparer.Equals(k, v_1)
        
        return (True, h, find_index(arrow_31, match_value[1]))
    
    else: 
        return (False, h, -1)
    


def HashSet__TryFind_2B595(this: HashSet[_T], k: _T) -> Optional[_T]:
    match_value : Tuple[bool, int, int] = HashSet__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return some(get_item_from_dict(this.hash_map, match_value[1])[match_value[2]])
    
    elif pattern_matching_result == 1:
        return None
    


def HashSet__get_Comparer(this: HashSet[_T]) -> IEqualityComparer[Any]:
    return this.comparer


def HashSet__Clear(this: HashSet[_T]) -> None:
    this.hash_map.clear()


def HashSet__get_Count(this: HashSet[_T]) -> int:
    count : int = 0
    enumerator : Any = get_enumerator(this.hash_map.values())
    try: 
        while enumerator.System_Collections_IEnumerator_MoveNext():
            items : List[_T] = enumerator.System_Collections_Generic_IEnumerator_00601_get_Current()
            count = (count + len(items)) or 0
    
    finally: 
        dispose(enumerator)
    
    return count


def HashSet__Add_2B595(this: HashSet[_T], k: _T) -> bool:
    match_value : Tuple[bool, int, int] = HashSet__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return False
    
    elif pattern_matching_result == 1:
        if match_value[0]:
            value : None = (get_item_from_dict(this.hash_map, match_value[1]).append(k))
            ignore()
            return True
        
        else: 
            this.hash_map[match_value[1]] = [k]
            return True
        
    


def HashSet__Contains_2B595(this: HashSet[_T], k: _T) -> bool:
    match_value : Tuple[bool, int, int] = HashSet__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return True
    
    elif pattern_matching_result == 1:
        return False
    


def HashSet__Remove_2B595(this: HashSet[_T], k: _T) -> bool:
    match_value : Tuple[bool, int, int] = HashSet__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        get_item_from_dict(this.hash_map, match_value[1]).pop(match_value[2])
        return True
    
    elif pattern_matching_result == 1:
        return False
    





from __future__ import annotations
from abc import abstractmethod
from typing import (Protocol, Callable, Optional)

class Event(Protocol):
    @property
    @abstractmethod
    def x(self) -> int:
        ...
    
    @property
    @abstractmethod
    def y(self) -> int:
        ...
    

class Misc(Protocol):
    @abstractmethod
    def bind(self, sequence: str, func: Callable[[Event], None]) -> Optional[str]:
        ...
    

class Wm(Protocol):
    pass




from __future__ import annotations
from abc import abstractmethod
import builtins
from typing import (Protocol, Any, Optional, Tuple, Callable, TypeVar)
from ...fable_library.util import IDisposable

_A_ = TypeVar("_A_")

class TextIOBase(Protocol):
    pass

class TextIOWrapper(TextIOBase, IDisposable):
    pass

class IExports(Protocol):
    @abstractmethod
    def chr(self, __arg0: int) -> str:
        ...
    
    @abstractmethod
    def float(self, __arg0: Any) -> float:
        ...
    
    @abstractmethod
    def id(self, __arg0: Any) -> int:
        ...
    
    @abstractmethod
    def int(self, __arg0: Any) -> int:
        ...
    
    @abstractmethod
    def len(self, __arg0: Any) -> int:
        ...
    
    @abstractmethod
    def open(self, file: Any, mode: Optional[Any], buffering: Optional[int], encoding: Optional[str], errors: Optional[str], newline: Optional[str], closefd: Optional[bool], opener: Optional[Callable[[Tuple[str, int]], int]]) -> TextIOWrapper:
        ...
    
    @abstractmethod
    def ord(self, __arg0: str) -> int:
        ...
    
    @abstractmethod
    def print(self, obj: Any) -> None:
        ...
    
    @abstractmethod
    def str(self, __arg0: Any) -> str:
        ...
    

def print(obj: Optional[_A_]=None) -> None:
    builtins.print(obj)





from typing import Any

from .types import FSharpRef


def parse(value: Any) -> float:
    try:
        return float(value)
    except Exception:
        raise ValueError("Input string was not in a correct format.")


def try_parse(string: str, def_value: FSharpRef[float]) -> bool:
    try:
        def_value.contents = parse(string)
        return True
    except Exception:
        return False



from __future__ import annotations
from typing import (TypeVar, Any, Optional, List, Tuple, Callable, Generic)
from .array import find_index
from .map_util import (try_get_value, get_item_from_dict)
from .reflection import (TypeInfo, class_type)
from .seq import (concat, iterate_indexed, to_array, delay, map, iterate)
from .string import format
from .types import FSharpRef
from .util import (get_enumerator, IEnumerator, to_iterator, equals, ignore, IEnumerable, ICollection, IEqualityComparer, dispose)

_KEY = TypeVar("_KEY")

_VALUE = TypeVar("_VALUE")

def expr_36(gen0: TypeInfo, gen1: TypeInfo) -> TypeInfo:
    return class_type("Fable.Collections.Dictionary", [gen0, gen1], Dictionary)


class Dictionary(Generic[_KEY, _VALUE]):
    def __init__(self, pairs: IEnumerable[Any], comparer: IEqualityComparer[Any]) -> None:
        this : FSharpRef[Dictionary[_KEY, _VALUE]] = FSharpRef(None)
        self.comparer = comparer
        this.contents = self
        self.hash_map = dict([])
        self.init_00409 = 1
        with get_enumerator(pairs) as enumerator:
            while enumerator.System_Collections_IEnumerator_MoveNext():
                pair : Any = enumerator.System_Collections_Generic_IEnumerator_00601_get_Current()
                Dictionary__Add_5BDDA1(this.contents, pair[0], pair[1])
    
    @property
    def Symbol_toStringTag(self) -> str:
        return "Dictionary"
    
    def to_json(self, _key: str) -> Any:
        this : Dictionary[_KEY, _VALUE] = self
        return list(this)
    
    def System_Collections_IEnumerable_GetEnumerator(self) -> IEnumerator[Any]:
        this : Dictionary[_KEY, _VALUE] = self
        return get_enumerator(this)
    
    def GetEnumerator(self) -> IEnumerator[Any]:
        this : Dictionary[_KEY, _VALUE] = self
        return get_enumerator(concat(this.hash_map.values()))
    
    def __iter__(self) -> IEnumerator[Any]:
        return to_iterator(self.GetEnumerator())
    
    def System_Collections_Generic_ICollection_00601_Add2B595(self, item: Any) -> None:
        this : Dictionary[_KEY, _VALUE] = self
        Dictionary__Add_5BDDA1(this, item[0], item[1])
    
    def System_Collections_Generic_ICollection_00601_Clear(self) -> None:
        this : Dictionary[_KEY, _VALUE] = self
        Dictionary__Clear(this)
    
    def System_Collections_Generic_ICollection_00601_Contains2B595(self, item: Any) -> bool:
        this : Dictionary[_KEY, _VALUE] = self
        match_value : Optional[Any] = Dictionary__TryFind_2B595(this, item[0])
        (pattern_matching_result,) = (None,)
        if match_value is not None:
            if equals(match_value[1], item[1]):
                pattern_matching_result = 0
            
            else: 
                pattern_matching_result = 1
            
        
        else: 
            pattern_matching_result = 1
        
        if pattern_matching_result == 0:
            return True
        
        elif pattern_matching_result == 1:
            return False
        
    
    def System_Collections_Generic_ICollection_00601_CopyToZ2E171D71(self, array: List[Any], array_index: int) -> None:
        this : Dictionary[_KEY, _VALUE] = self
        def action(i: int, e: Any) -> None:
            array[array_index + i] = e
        
        iterate_indexed(action, this)
    
    def System_Collections_Generic_ICollection_00601_get_Count(self) -> int:
        this : Dictionary[_KEY, _VALUE] = self
        return Dictionary__get_Count(this)
    
    def System_Collections_Generic_ICollection_00601_get_IsReadOnly(self) -> bool:
        return False
    
    def System_Collections_Generic_ICollection_00601_Remove2B595(self, item: Any) -> bool:
        this : Dictionary[_KEY, _VALUE] = self
        match_value : Optional[Any] = Dictionary__TryFind_2B595(this, item[0])
        if match_value is not None:
            if equals(match_value[1], item[1]):
                ignore(Dictionary__Remove_2B595(this, item[0]))
            
            return True
        
        else: 
            return False
        
    
    def System_Collections_Generic_IDictionary_00602_Add5BDDA1(self, key: _KEY, value: _VALUE) -> None:
        this : Dictionary[_KEY, _VALUE] = self
        Dictionary__Add_5BDDA1(this, key, value)
    
    def System_Collections_Generic_IDictionary_00602_ContainsKey2B595(self, key: Optional[_KEY]=None) -> bool:
        this : Dictionary[_KEY, _VALUE] = self
        return Dictionary__ContainsKey_2B595(this, key)
    
    def System_Collections_Generic_IDictionary_00602_get_Item2B595(self, key: Optional[_KEY]=None) -> _VALUE:
        this : Dictionary[_KEY, _VALUE] = self
        return Dictionary__get_Item_2B595(this, key)
    
    def System_Collections_Generic_IDictionary_00602_set_Item5BDDA1(self, key: _KEY, v: _VALUE) -> None:
        this : Dictionary[_KEY, _VALUE] = self
        Dictionary__set_Item_5BDDA1(this, key, v)
    
    def System_Collections_Generic_IDictionary_00602_get_Keys(self) -> ICollection[_KEY]:
        this : Dictionary[_KEY, _VALUE] = self
        def arrow_33(__unit: Any=None) -> IEnumerable[_KEY]:
            def arrow_32(pair: Any) -> _KEY:
                return pair[0]
            
            return map(arrow_32, this)
        
        return to_array(delay(arrow_33))
    
    def System_Collections_Generic_IDictionary_00602_Remove2B595(self, key: Optional[_KEY]=None) -> bool:
        this : Dictionary[_KEY, _VALUE] = self
        return Dictionary__Remove_2B595(this, key)
    
    def System_Collections_Generic_IDictionary_00602_TryGetValue6DC89625(self, key: _KEY, value: FSharpRef[_VALUE]) -> bool:
        this : Dictionary[_KEY, _VALUE] = self
        match_value : Optional[Any] = Dictionary__TryFind_2B595(this, key)
        if match_value is not None:
            pair : Any = match_value
            value.contents = pair[1]
            return True
        
        else: 
            return False
        
    
    def System_Collections_Generic_IDictionary_00602_get_Values(self) -> ICollection[_VALUE]:
        this : Dictionary[_KEY, _VALUE] = self
        def arrow_35(__unit: Any=None) -> IEnumerable[_VALUE]:
            def arrow_34(pair: Any) -> _VALUE:
                return pair[1]
            
            return map(arrow_34, this)
        
        return to_array(delay(arrow_35))
    
    @property
    def size(self) -> int:
        this : Dictionary[_KEY, _VALUE] = self
        return Dictionary__get_Count(this)
    
    def clear(self) -> None:
        this : Dictionary[_KEY, _VALUE] = self
        Dictionary__Clear(this)
    
    def delete(self, k: Optional[_KEY]=None) -> bool:
        this : Dictionary[_KEY, _VALUE] = self
        return Dictionary__Remove_2B595(this, k)
    
    def entries(self) -> IEnumerable[Tuple[_KEY, _VALUE]]:
        this : Dictionary[_KEY, _VALUE] = self
        def mapping(p: Any) -> Tuple[_KEY, _VALUE]:
            return (p[0], p[1])
        
        return map(mapping, this)
    
    def __getitem__(self, k: Optional[_KEY]=None) -> _VALUE:
        this : Dictionary[_KEY, _VALUE] = self
        return Dictionary__get_Item_2B595(this, k)
    
    def has(self, k: Optional[_KEY]=None) -> bool:
        this : Dictionary[_KEY, _VALUE] = self
        return Dictionary__ContainsKey_2B595(this, k)
    
    def keys(self) -> IEnumerable[_KEY]:
        this : Dictionary[_KEY, _VALUE] = self
        def mapping(p: Any) -> _KEY:
            return p[0]
        
        return map(mapping, this)
    
    def __setitem__(self, k: _KEY, v: _VALUE) -> Map_2[_KEY, _VALUE]:
        this : Dictionary[_KEY, _VALUE] = self
        Dictionary__set_Item_5BDDA1(this, k, v)
        return this
    
    def values(self) -> IEnumerable[_VALUE]:
        this : Dictionary[_KEY, _VALUE] = self
        def mapping(p: Any) -> _VALUE:
            return p[1]
        
        return map(mapping, this)
    
    def for_each(self, f: Callable[[_VALUE, _KEY, Map_2[_KEY, _VALUE]], None], this_arg: Optional[Any]=None) -> None:
        this : Dictionary[_KEY, _VALUE] = self
        def action(p: Any) -> None:
            f(p[1], p[0], this)
        
        iterate(action, this)
    

Dictionary_reflection = expr_36

def Dictionary__ctor_6623D9B3(pairs: IEnumerable[Any], comparer: IEqualityComparer[Any]) -> Dictionary[_KEY, _VALUE]:
    return Dictionary(pairs, comparer)


def Dictionary__TryFindIndex_2B595(this: Dictionary[_KEY, _VALUE], k: _KEY) -> Tuple[bool, int, int]:
    h : int = this.comparer.GetHashCode(k) or 0
    match_value : Tuple[bool, List[Any]]
    out_arg : List[Any] = None
    def arrow_37(this: Dictionary[_KEY, _VALUE]=this, k: _KEY=k) -> List[Any]:
        return out_arg
    
    def arrow_38(v: List[Any], this: Dictionary[_KEY, _VALUE]=this, k: _KEY=k) -> None:
        nonlocal out_arg
        out_arg = v
    
    match_value = (try_get_value(this.hash_map, h, FSharpRef(arrow_37, arrow_38)), out_arg)
    if match_value[0]:
        def arrow_39(pair: Any, this: Dictionary[_KEY, _VALUE]=this, k: _KEY=k) -> bool:
            return this.comparer.Equals(k, pair[0])
        
        return (True, h, find_index(arrow_39, match_value[1]))
    
    else: 
        return (False, h, -1)
    


def Dictionary__TryFind_2B595(this: Dictionary[_KEY, _VALUE], k: _KEY) -> Optional[Any]:
    match_value : Tuple[bool, int, int] = Dictionary__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return get_item_from_dict(this.hash_map, match_value[1])[match_value[2]]
    
    elif pattern_matching_result == 1:
        return None
    


def Dictionary__get_Comparer(this: Dictionary[_KEY, _VALUE]) -> IEqualityComparer[Any]:
    return this.comparer


def Dictionary__Clear(this: Dictionary[_KEY, _VALUE]) -> None:
    this.hash_map.clear()


def Dictionary__get_Count(this: Dictionary[_KEY, _VALUE]) -> int:
    count : int = 0
    enumerator : Any = get_enumerator(this.hash_map.values())
    try: 
        while enumerator.System_Collections_IEnumerator_MoveNext():
            pairs : List[Any] = enumerator.System_Collections_Generic_IEnumerator_00601_get_Current()
            count = (count + len(pairs)) or 0
    
    finally: 
        dispose(enumerator)
    
    return count


def Dictionary__get_Item_2B595(this: Dictionary[_KEY, _VALUE], k: _KEY) -> _VALUE:
    match_value : Optional[Any] = Dictionary__TryFind_2B595(this, k)
    if match_value is not None:
        return match_value[1]
    
    else: 
        raise Exception("The item was not found in collection")
    


def Dictionary__set_Item_5BDDA1(this: Dictionary[_KEY, _VALUE], k: _KEY, v: _VALUE) -> None:
    match_value : Tuple[bool, int, int] = Dictionary__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        get_item_from_dict(this.hash_map, match_value[1])[match_value[2]] = (k, v)
    
    elif pattern_matching_result == 1:
        if match_value[0]:
            value : None = (get_item_from_dict(this.hash_map, match_value[1]).append((k, v)))
            ignore()
        
        else: 
            this.hash_map[match_value[1]] = [(k, v)]
        
    


def Dictionary__Add_5BDDA1(this: Dictionary[_KEY, _VALUE], k: _KEY, v: _VALUE) -> None:
    match_value : Tuple[bool, int, int] = Dictionary__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        raise Exception(format("An item with the same key has already been added. Key: {0}", k))
    
    elif pattern_matching_result == 1:
        if match_value[0]:
            value : None = (get_item_from_dict(this.hash_map, match_value[1]).append((k, v)))
            ignore()
        
        else: 
            this.hash_map[match_value[1]] = [(k, v)]
        
    


def Dictionary__ContainsKey_2B595(this: Dictionary[_KEY, _VALUE], k: _KEY) -> bool:
    match_value : Tuple[bool, int, int] = Dictionary__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return True
    
    elif pattern_matching_result == 1:
        return False
    


def Dictionary__Remove_2B595(this: Dictionary[_KEY, _VALUE], k: _KEY) -> bool:
    match_value : Tuple[bool, int, int] = Dictionary__TryFindIndex_2B595(this, k)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[2] > -1:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        get_item_from_dict(this.hash_map, match_value[1]).pop(match_value[2])
        return True
    
    elif pattern_matching_result == 1:
        return False
    





from __future__ import annotations
from ...fs_hafas_python.context import Context
from ...fs_hafas_python.lib.slug import slugify
from ...fs_hafas_python.types_hafas_client import Operator
from ...fs_hafas_python.types_raw_hafas_client import RawOp

def slug(s: str) -> str:
    return slugify(s)


default_operator : Operator = Operator("operator", "", "")

def parse_operator(ctx: Context, a: RawOp) -> Operator:
    name : str = a.name.strip()
    return Operator(default_operator.type, slug(name), name)





from __future__ import annotations
from datetime import timedelta
from typing import (ByteString, Any, List, Optional, Tuple)
from ...fable_library.array import (map, fold as fold_1, sum_by, choose)
from ...fable_library.date import (create, to_string)
from ...fable_library.int32 import parse
from ...fable_library.option import default_arg
from ...fable_library.range import range_double
from ...fable_library.seq import (to_array, fold)
from ...fable_library.string import substring
from ...fable_library.types import Uint8Array
from ...fs_hafas_python.context import (Context, Profile__get_parseJourneyLeg)
from ...fs_hafas_python.extra_types import (IndexMap_2, IndexMap_2__set_Item_541DA560, IndexMap_2__ctor_2B594)
from ...fs_hafas_python.lib.transformations import Default_Journey
from ...fs_hafas_python.types_hafas_client import (Leg, Cycle, Journey, FeatureCollection)
from ...fs_hafas_python.types_raw_hafas_client import (RawOutCon, RawSec, RawFreq)
from .common import msg_lto_remarks
from .polyline import distance_of_feature_collection

def mapping(x: int) -> int:
    value : int = (1 << (7 - x)) or 0
    return int(value+0x100 if value < 0 else value) & 0xFF


bytes : ByteString = map(mapping, to_array(range_double(0, 1, 7)), Uint8Array)

def parse_scheduled_days(ctx: Context, s_days: str, year: int) -> IndexMap_2[str, bool]:
    dt : Any = create(year, 1, 1)
    source : ByteString = bytes.fromhex(s_days)
    def folder_1(m: IndexMap_2[str, bool], d: int, ctx: Context=ctx, s_days: str=s_days, year: int=year) -> IndexMap_2[str, bool]:
        def folder(m_1: IndexMap_2[str, bool], b: int, m: IndexMap_2[str, bool]=m, d: int=d) -> IndexMap_2[str, bool]:
            nonlocal dt
            IndexMap_2__set_Item_541DA560(m_1, to_string(dt, "yyyy-MM-dd"), (d & b) != 0)
            dt = dt+timedelta(days=1)
            return m_1
        
        return fold_1(folder, m, bytes)
    
    return fold(folder_1, IndexMap_2__ctor_2B594(False), source)


def parse_journey(ctx: Context, j: RawOutCon) -> Journey:
    def mapping(l: RawSec, ctx: Context=ctx, j: RawOutCon=j) -> Leg:
        return Profile__get_parseJourneyLeg(ctx.profile)(ctx)(l)(j.date)
    
    legs : List[Leg] = map(mapping, j.sec_l, None)
    def arrow_438(ctx: Context=ctx, j: RawOutCon=j) -> List[Any]:
        value : List[Any] = []
        return default_arg(msg_lto_remarks(ctx, j.msg_l), value)
    
    remarks : Optional[List[Any]] = arrow_438() if ctx.opt.remarks else None
    scheduled_days : Optional[IndexMap_2[str, bool]]
    match_value : Tuple[bool, Optional[str]] = (ctx.opt.scheduled_days, j.s_days.s_days_b)
    (pattern_matching_result,) = (None,)
    if match_value[0]:
        if match_value[1] is not None:
            pattern_matching_result = 0
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        scheduled_days = parse_scheduled_days(ctx, match_value[1], parse(substring(j.date, 0, 4), 511, False, 32))
    
    elif pattern_matching_result == 1:
        scheduled_days = None
    
    def arrow_441(ctx: Context=ctx, j: RawOutCon=j) -> Optional[Cycle]:
        match_value_1 : Optional[RawFreq] = j.freq
        if match_value_1 is None:
            return None
        
        else: 
            freq : RawFreq = match_value_1
            match_value_2 : Tuple[Optional[int], Optional[int]] = (freq.min_c, freq.max_c)
            def arrow_439(__unit: Any=None) -> Optional[Cycle]:
                max_c : int = match_value_2[1] or 0
                min_c : int = match_value_2[0] or 0
                return Cycle(min_c * 60, max_c * 60, freq.num_c)
            
            def arrow_440(__unit: Any=None) -> Optional[Cycle]:
                min_c_1 : int = match_value_2[0] or 0
                return Cycle(min_c_1 * 60, None, None)
            
            return (arrow_439() if (match_value_2[1] is not None) else arrow_440()) if (match_value_2[0] is not None) else None
        
    
    return Journey(Default_Journey.type, legs, j.ctx_recon, remarks, Default_Journey.price, arrow_441(), scheduled_days)


def distance_of_journey(j: Journey) -> float:
    def arrow_442(fc: FeatureCollection, j: Journey=j) -> float:
        return distance_of_feature_collection(fc)
    
    def chooser(l: Leg, j: Journey=j) -> Optional[FeatureCollection]:
        return l.polyline
    
    class ObjectExpr445:
        @property
        def GetZero(self) -> Any:
            def arrow_443(__unit: Any=None) -> int:
                return 0
            
            return arrow_443
        
        @property
        def Add(self) -> Any:
            def arrow_444(x: float, y: float) -> float:
                return x + y
            
            return arrow_444
        
    return sum_by(arrow_442, choose(chooser, j.legs, None), ObjectExpr445())





from abc import abstractmethod
from typing import (ByteString, Protocol)

class IExports(Protocol):
    @abstractmethod
    def b16encode(self, __arg0: ByteString) -> int:
        ...
    
    @abstractmethod
    def b32encode(self, __arg0: ByteString) -> ByteString:
        ...
    
    @abstractmethod
    def standard_b64encode(self, __arg0: ByteString) -> ByteString:
        ...
    
    @abstractmethod
    def urlsafe_b64encode(self, __arg0: ByteString) -> ByteString:
        ...
    




from datetime import timedelta
from typing import Optional


def total_seconds(ts: timedelta) -> float:
    return ts.total_seconds()


def from_milliseconds(msecs: int) -> timedelta:
    return timedelta(milliseconds=msecs)


def from_ticks(ticks: int) -> timedelta:
    return timedelta(microseconds=ticks // 10)


def to_milliseconds(td: timedelta) -> int:
    return int(td.total_seconds() * 1000)


def create(
    d: int = 0,
    h: Optional[int] = None,
    m: Optional[int] = None,
    s: Optional[int] = None,
    ms: Optional[int] = None,
) -> timedelta:
    if h is None and m is None and s is None and ms is None:
        return from_ticks(d)

    return timedelta(
        days=d, hours=h or 0, minutes=m or 0, seconds=s or 0, milliseconds=ms or 0
    )



from __future__ import annotations
from typing import (Optional, Tuple, Any, Callable, List)
from ...fable_library.array import (filter as filter_1, try_head)
from ...fable_library.option import (value as value_1, map, default_arg, to_array)
from ...fable_library.seq import fold
from ...fs_hafas_python.context import (Context, Profile__get_parseWhen, ParsedWhen, Profile__get_parsePlatform, Platform, Profile__get_parseStopovers, Profile__get_parseLocations)
from ...fs_hafas_python.lib.transformations import (RawDep_FromRawStopL, RawArr_FromRawStopL, U2StationStop_FromSomeU3StationStopLocation, Default_Location, Default_Alternative)
from ...fs_hafas_python.types_hafas_client import (StopOver, Location, Alternative)
from ...fs_hafas_python.types_raw_hafas_client import (RawJny, RawPltf, RawDep, RawArr, RawCrd, RawProd, RawCommon, RawLoc, RawMsg, RawStop)
from .common import (get_element_at_some, msg_lto_remarks, append_some_array, get_element_at)

DEP : str = "DEP"

ARR : str = "ARR"

def parse_departure_arrival(type: str, ctx: Context, d: RawJny) -> Alternative:
    pattern_input : Tuple[Optional[int], Optional[str], Optional[str], Optional[int], Optional[bool], Optional[str], Optional[str], Optional[RawPltf], Optional[RawPltf]]
    if type == DEP:
        dep : RawDep = RawDep_FromRawStopL(value_1(d.stb_stop))
        pattern_input = (dep.loc_x, dep.d_time_s, dep.d_time_r, dep.d_tzoffset, dep.d_cncl, dep.d_platf_s, dep.d_platf_r, dep.d_pltf_s, dep.d_pltf_r)
    
    else: 
        arr : RawArr = RawArr_FromRawStopL(value_1(d.stb_stop))
        pattern_input = (arr.loc_x, arr.a_time_s, arr.a_time_r, arr.a_tzoffset, arr.a_cncl, arr.a_platf_s, arr.a_platf_r, arr.a_pltf_s, arr.a_pltf_r)
    
    x_cncl : Optional[bool] = pattern_input[4]
    stop : Optional[Any] = U2StationStop_FromSomeU3StationStopLocation(get_element_at_some(pattern_input[0], ctx.common.locations))
    w : ParsedWhen = Profile__get_parseWhen(ctx.profile)(ctx)(value_1(d.date))(pattern_input[1])(pattern_input[2])(pattern_input[3])(x_cncl)
    def match_platf_s(a_platf_s: Optional[str]=None, a_pltf_s: Optional[RawPltf]=None, type: str=type, ctx: Context=ctx, d: RawJny=d) -> Optional[str]:
        if a_platf_s is None:
            if a_pltf_s is not None:
                return a_pltf_s.txt
            
            else: 
                return None
            
        
        else: 
            return a_platf_s
        
    
    platf_s_1 : Optional[str] = match_platf_s(pattern_input[5], pattern_input[7])
    platf_r : Optional[str] = match_platf_s(pattern_input[6], pattern_input[8])
    plt : Platform = Profile__get_parsePlatform(ctx.profile)(ctx)(platf_s_1)(platf_r)(x_cncl)
    def mapping(st: List[StopOver], type: str=type, ctx: Context=ctx, d: RawJny=d) -> List[StopOver]:
        def filter(s: StopOver, st: List[StopOver]=st) -> bool:
            match_value : Optional[bool] = s.pass_by
            if match_value is None:
                return True
            
            else: 
                return not match_value
            
        
        return filter_1(filter, st)
    
    stopovers : Optional[List[StopOver]] = map(mapping, Profile__get_parseStopovers(ctx.profile)(ctx)(d.stop_l)(value_1(d.date)))
    current_trip_position : Optional[Location]
    match_value_1 : Optional[RawCrd] = d.pos
    if match_value_1 is None:
        current_trip_position = None
    
    else: 
        pos : RawCrd = match_value_1
        current_trip_position = Location(Default_Location.type, Default_Location.id, Default_Location.name, Default_Location.poi, Default_Location.address, pos.x / 1000000, pos.y / 1000000, Default_Location.altitude, Default_Location.distance)
    
    destination : Optional[Any]
    if type == DEP:
        match_value_2 : Tuple[Optional[List[RawProd]], Optional[RawCommon]] = (d.prod_l, ctx.res.common)
        (pattern_matching_result, common_1, prod_l_1) = (None, None, None)
        if match_value_2[0] is not None:
            if match_value_2[1] is not None:
                def arrow_365(type: str=type, ctx: Context=ctx, d: RawJny=d) -> bool:
                    common : RawCommon = match_value_2[1]
                    return (common.loc_l is not None) if (len(match_value_2[0]) > 0) else False
                
                if arrow_365():
                    pattern_matching_result = 0
                    common_1 = match_value_2[1]
                    prod_l_1 = match_value_2[0]
                
                else: 
                    pattern_matching_result = 1
                
            
            else: 
                pattern_matching_result = 1
            
        
        else: 
            pattern_matching_result = 1
        
        if pattern_matching_result == 0:
            loc : Optional[RawLoc] = get_element_at_some(prod_l_1[0].t_loc_x, value_1(common_1.loc_l))
            destination = try_head(Profile__get_parseLocations(ctx.profile)(ctx)([value_1(loc)]))
        
        elif pattern_matching_result == 1:
            destination = None
        
    
    else: 
        destination = None
    
    origin : Optional[Any]
    if type == ARR:
        match_value_3 : Tuple[Optional[List[RawProd]], Optional[RawCommon]] = (d.prod_l, ctx.res.common)
        (pattern_matching_result_1, common_3, prod_l_3) = (None, None, None)
        if match_value_3[0] is not None:
            if match_value_3[1] is not None:
                def arrow_367(type: str=type, ctx: Context=ctx, d: RawJny=d) -> bool:
                    common_2 : RawCommon = match_value_3[1]
                    return (common_2.loc_l is not None) if (len(match_value_3[0]) > 0) else False
                
                if arrow_367():
                    pattern_matching_result_1 = 0
                    common_3 = match_value_3[1]
                    prod_l_3 = match_value_3[0]
                
                else: 
                    pattern_matching_result_1 = 1
                
            
            else: 
                pattern_matching_result_1 = 1
            
        
        else: 
            pattern_matching_result_1 = 1
        
        if pattern_matching_result_1 == 0:
            loc_1 : Optional[RawLoc] = get_element_at_some(prod_l_3[0].f_loc_x, value_1(common_3.loc_l))
            origin = try_head(Profile__get_parseLocations(ctx.profile)(ctx)([value_1(loc_1)]))
        
        elif pattern_matching_result_1 == 1:
            origin = None
        
    
    else: 
        origin = None
    
    def arrow_371(type: str=type, ctx: Context=ctx, d: RawJny=d) -> List[Any]:
        value : List[Any] = []
        def folder(_arg1: Optional[List[RawMsg]], s_1: RawStop) -> Optional[List[RawMsg]]:
            return s_1.msg_l
        
        return default_arg(msg_lto_remarks(ctx, append_some_array(d.msg_l, fold(folder, None, to_array(d.stb_stop)))), value)
    
    remarks : Optional[List[Any]] = arrow_371() if ctx.opt.remarks else None
    return Alternative(d.jid, d.dir_txt, Default_Alternative.location, get_element_at(d.prod_x, ctx.common.lines), stop, w.when, w.planned_when, w.prognosed_when, w.delay, plt.platform, plt.planned_platform, plt.prognosed_platform, remarks, value_1(d.stb_stop).d_cncl, Default_Alternative.load_factor, None, Default_Alternative.previous_stopovers, stopovers, Default_Alternative.frames, Default_Alternative.polyline, current_trip_position, origin, destination)


def parse_departure(ctx: Context, d: RawJny) -> Alternative:
    return parse_departure_arrival(DEP, ctx, d)


def parse_arrival(ctx: Context, d: RawJny) -> Alternative:
    return parse_departure_arrival(ARR, ctx, d)





from typing import (Any, List)
from ..fable_library.reflection import (TypeInfo, float64_type, string_type, bool_type, list_type, class_type, union_type)
from ..fable_library.types import Union

def expr_3() -> TypeInfo:
    return union_type("Fable.SimpleJson.Python.Json", [], Json, lambda: [[("Item", float64_type)], [("Item", string_type)], [("Item", bool_type)], [], [("Item", list_type(Json_reflection()))], [("Item", class_type("Microsoft.FSharp.Collections.FSharpMap`2", [string_type, Json_reflection()]))]])


class Json(Union):
    def __init__(self, tag: int, *fields: Any) -> None:
        super().__init__()
        self.tag : int = tag or 0
        self.fields : List[Any] = list(fields)
    
    @staticmethod
    def cases() -> List[str]:
        return ["JNumber", "JString", "JBool", "JNull", "JArray", "JObject"]
    

Json_reflection = expr_3




from typing import Optional


def to_fixed(x: float, dp: Optional[int] = None) -> str:
    if dp is not None:
        fmt = "{:.%sf}" % dp
        return fmt.format(x)

    return "{}".format(x)


def to_precision(x: float, sd: Optional[int] = None) -> str:
    if sd is not None:
        fmt = "{:.%se}" % sd
        return fmt.format(x)

    return "{}".format(x)


def to_exponential(x: float, dp: Optional[int] = None) -> str:
    if dp is not None:
        fmt = "{:.%se}" % dp
        return fmt.format(x)

    return "{}".format(x)


def to_hex(x: int) -> str:
    return "{0:x}".format(x)


def multiply(x: int, y: int) -> int:
    return x * y



from __future__ import annotations
from typing import (Optional, Tuple)
from ...fable_library.array import try_find
from ...fable_library.option import (default_arg, bind)
from ...fable_library.seq import (filter, try_head, empty)
from ...fable_library.util import IEnumerable
from ...fs_hafas_python.context import (Context, Profile__get_parseBitmask)
from ...fs_hafas_python.extra_types import (IndexMap_2, IndexMap_2__get_Item_2B595, IndexMap_2__get_Keys)
from ...fs_hafas_python.lib.slug import slugify
from ...fs_hafas_python.lib.transformations import Default_Line
from ...fs_hafas_python.types_hafas_client import (Line, ProductType, Operator)
from ...fs_hafas_python.types_raw_hafas_client import (RawProd, RawProdCtx)
from .common import get_element_at_some

def slug(s: str) -> Optional[str]:
    return slugify(s)


def filter_products(products: IndexMap_2[str, bool]) -> IEnumerable[str]:
    def predicate(kv: str, products: IndexMap_2[str, bool]=products) -> bool:
        return IndexMap_2__get_Item_2B595(products, kv)
    
    return filter(predicate, IndexMap_2__get_Keys(products))


def parse_line(ctx: Context, p: RawProd) -> Line:
    line : Line = Default_Line
    name : Optional[str] = default_arg(p.add_name, p.name)
    id : Optional[str]
    match_value : Optional[RawProdCtx] = p.prod_ctx
    def arrow_422(s: str, ctx: Context=ctx, p: RawProd=p) -> Optional[str]:
        return slug(s)
    
    id = (None if (name is None) else slug(name)) if (match_value is None) else bind(arrow_422, default_arg(match_value.line_id, name))
    pattern_input : Tuple[Optional[str], Optional[str], Optional[str]]
    match_value_1 : Optional[RawProdCtx] = p.prod_ctx
    if match_value_1 is None:
        pattern_input = (None, None, None)
    
    else: 
        prod_ctx_1 : RawProdCtx = match_value_1
        pattern_input = (prod_ctx_1.num, prod_ctx_1.admin, prod_ctx_1.cat_out)
    
    cat_out : Optional[str] = pattern_input[2]
    def arrow_423(ctx: Context=ctx, p: RawProd=p) -> Optional[str]:
        cat_out_2 : str = cat_out
        return cat_out_2.strip()
    
    product_name : Optional[str] = (arrow_423() if (cat_out != "") else None) if (cat_out is not None) else None
    def arrow_424(ctx: Context=ctx, p: RawProd=p) -> IEnumerable[str]:
        match_value_2 : Optional[int] = p.cls
        if match_value_2 is None:
            return empty()
        
        else: 
            cls : int = match_value_2 or 0
            return filter_products(Profile__get_parseBitmask(ctx.profile)(ctx)(cls))
        
    
    product : Optional[str] = try_head(arrow_424())
    pattern_input_1 : Tuple[Optional[str], Optional[str]]
    if product is None:
        pattern_input_1 = (None, None)
    
    else: 
        kv : str = product
        def predicate(p_1: ProductType, ctx: Context=ctx, p: RawProd=p) -> bool:
            return p_1.id == kv
        
        match_value_3 : Optional[ProductType] = try_find(predicate, ctx.profile.products)
        if match_value_3 is None:
            pattern_input_1 = (None, None)
        
        else: 
            product_1 : ProductType = match_value_3
            pattern_input_1 = (product_1.id, product_1.mode)
        
    
    operator : Optional[Operator] = get_element_at_some(p.opr_x, ctx.common.operators)
    return Line(line.type, id, name, pattern_input[1], pattern_input[0], line.additional_name, pattern_input_1[0], True, pattern_input_1[1], line.routes, operator, line.express, line.metro, line.night, line.nr, line.symbol, line.directions, product_name)





from __future__ import annotations

from queue import SimpleQueue
from threading import RLock
from typing import Any, Callable, Generic, List, Optional, TypeVar

from .async_ import from_continuations, start_immediate
from .async_builder import Async, CancellationToken, OperationCanceledError

_Msg = TypeVar("_Msg")
_Reply = TypeVar("_Reply")


class AsyncReplyChannel(Generic[_Reply]):
    def __init__(self, fn: Callable[[Any], None]) -> None:
        self.fn = fn

    def reply(self, r: Any) -> None:
        self.fn(r)


class MailboxProcessor(Generic[_Msg]):
    def __init__(
        self,
        body: Callable[[MailboxProcessor[_Msg]], Async[None]],
        cancellation_token: Optional[CancellationToken] = None,
    ):
        self.messages: SimpleQueue[_Msg] = SimpleQueue()
        self.token = cancellation_token or CancellationToken()
        self.lock = RLock()
        self.body = body

        # Holds the continuation i.e the `done` callback of Async.from_continuations returned by `receive`.
        self.continuation: List[Callable[[Any], None]] = []

    def post(self, msg: _Msg) -> None:
        """Post a message synchronously to the mailbox processor.

        This method is not asynchronous since it's very fast to execute.
        It simply adds the message to the message queue of the mailbox
        processor and returns.

        Args:
            msg: Message to post.

        Returns:
            None
        """
        self.messages.put(msg)
        self.__process_events()

    def post_and_async_reply(
        self, build_message: Callable[[AsyncReplyChannel[_Reply]], _Msg]
    ) -> Async[_Reply]:
        """Post a message asynchronously to the mailbox processor and
        wait for the reply.

        Args:
            build_message: A function that takes a reply channel
            (`AsyncReplyChannel[Reply]`) and returns a message to send
            to the mailbox processor. The message should contain the
            reply channel as e.g a tuple.

        Returns:
            The reply from mailbox processor.
        """

        result = None
        continuation = None  # This is the continuation for the `done` callback of the awaiting poster.

        def check_completion() -> None:
            if result is not None and continuation is not None:
                continuation[0](result)

        def reply_callback(res: _Reply):
            nonlocal result
            result = res
            check_completion()

        reply_channel: AsyncReplyChannel[_Reply] = AsyncReplyChannel(reply_callback)
        self.messages.put(build_message(reply_channel))
        self.__process_events()

        def callback(conts: List[Callable[[Any], None]]) -> None:
            nonlocal continuation
            continuation = conts
            check_completion()

        return from_continuations(callback)

    def receive(self) -> Async[_Msg]:
        """Receive message from mailbox.

        Returns:
            An asynchronous computation which will consume the
            first message in arrival order. No thread is blocked while
            waiting for further messages. Raises a TimeoutException if
            the timeout is exceeded.
        """

        def callback(conts: List[Callable[[Any], None]]):
            if self.continuation:
                raise Exception("Receive can only be called once!")

            self.continuation = conts

            self.__process_events()

        return from_continuations(callback)

    def __process_events(self) -> None:
        if not self.continuation:
            return

        # Cancellation of async workflows is more tricky in Python than
        # with F# so we check the cancellation token for each process.
        if self.token.is_cancellation_requested:
            self.continuation, cont = [], self.continuation
            if cont:
                cont[2](OperationCanceledError("Mailbox was cancelled"))
            return

        with self.lock:
            if self.messages.empty():
                return
            msg = self.messages.get()
            self.continuation, cont = [], self.continuation

        if cont:
            cont[0](msg)

    @classmethod
    def start(
        cls,
        body: Callable[[MailboxProcessor[_Msg]], Async[None]],
        cancellation_token: Optional[CancellationToken] = None,
    ) -> MailboxProcessor[_Msg]:
        mbox: MailboxProcessor[_Msg] = MailboxProcessor(body, cancellation_token)
        start_immediate(body(mbox))
        return mbox


def receive(mbox: MailboxProcessor[_Msg]) -> Async[_Msg]:
    return mbox.receive()


def post(mbox: MailboxProcessor[_Msg], msg: _Msg):
    return mbox.post(msg)


def start_instance(mbox: MailboxProcessor[Any]) -> None:
    body = mbox.body(mbox)
    return start_immediate(body)


def start(
    body: Callable[[MailboxProcessor[_Msg]], Async[None]],
    cancellationToken: Optional[CancellationToken] = None,
) -> MailboxProcessor[_Msg]:
    mbox = MailboxProcessor(body, cancellationToken)
    start_instance(mbox)
    return mbox


__all__ = [
    "AsyncReplyChannel",
    "MailboxProcessor",
    "receive",
    "post",
    "start",
    "start_instance",
]



from typing import (MutableSequence, List, Callable)
from ...fable_library.reflection import (TypeInfo, string_type, float64_type, array_type, lambda_type, record_type)
from ...fable_library.types import Record

def expr_289() -> TypeInfo:
    return record_type("FsHafas.Extensions.PolylineEx.GooglePolyline", [], GooglePolyline, lambda: [("decode", lambda_type(string_type, array_type(array_type(float64_type))))])


class GooglePolyline(Record):
    def __init__(self, decode: Callable[[str], List[MutableSequence[float]]]=None) -> None:
        super().__init__()
        self.decode = decode
    

GooglePolyline_reflection = expr_289




from enum import Enum
from typing import Optional
from urllib.parse import urlparse


class UriKind(Enum):
    RelativeOrAbsolute = 0
    Absolute = 1
    Relative = 2


class Uri:
    def __init__(self, uri: str, uri_kind: Optional[int] = None) -> None:
        self.res = urlparse(uri)
        self.kind = UriKind(uri_kind) if uri_kind else UriKind.Absolute

    @property
    def is_absolute_uri(self) -> bool:
        return True if self.res.netloc else False

    @property
    def scheme(self) -> str:
        return self.res.scheme

    @property
    def host(self) -> str:
        return self.res.hostname or ""

    @property
    def absolute_uri(self) -> str:
        if self.kind == UriKind.Relative:
            raise ValueError("This operation is not supported for a relative URI.")

        return self.res.geturl()

    @property
    def absolute_path(self) -> str:
        return self.res.path

    @property
    def query(self) -> str:
        return f"?{self.res.query}"

    @property
    def path_and_query(self) -> str:
        return f"{self.res.path}?{self.res.query}"

    @property
    def fragment(self) -> str:
        return f"#{self.res.fragment}"

    def __str__(self) -> str:
        return self.res.geturl()



from __future__ import annotations
from array import array as array_2
from typing import (List, Tuple, Optional, Any, MutableSequence, TypeVar, Callable)
from ...fable_library.array import (try_find, map, append)
from ...fable_library.option import (bind, value as value_2, some, default_arg)
from ...fable_library.reflection import (TypeInfo, string_type, record_type)
from ...fable_library.reg_exp import is_match
from ...fable_library.types import Record
from ...fable_library.util import round
from ...fs_hafas_python.context import (Options, Context, Profile, Profile__set__locale_Z721C83C5, Profile__set__timezone_Z721C83C5, Profile__set__endpoint_Z721C83C5, Profile__set_salt_Z721C83C5, Profile__set_cfg_Z3219B2F8, Profile__set_baseRequest_Z42C91061, Profile__set__products_76A34681, Profile__set__trip_6FCE9E49, Profile__set__radar_6FCE9E49, Profile__set__tripsByName_6FCE9E49, Profile__set__reachableFrom_6FCE9E49, Profile__set__journeysFromTrip_6FCE9E49, Profile__set_journeysOutFrwd_Z1FBCCD16, Profile__set_formatStation_11D407F6, Profile__set_transformJourneysQuery_4AA4AF64, Profile__get_parseJourney, Profile__set_parseJourney_Z1F35F4C, Profile__get_parseJourneyLeg, Profile__set_parseJourneyLeg_3913217E, Profile__get_parseDeparture, Profile__set_parseDeparture_537DC5A, Profile__get_parseHint, Profile__set_parseHint_2044943E, Profile__get_parseLine, Profile__set_parseLine_718F82F)
from ...fs_hafas_python.lib.transformations import Default_JourneysOptions
from ...fs_hafas_python.profile import default_profile
from ...fs_hafas_python.types_hafas_client import (ProductType, Hint, Status, Line, Leg, Alternative, Journey, Price, LoyaltyCard, JourneysOptions)
from ...fs_hafas_python.types_raw_hafas_client import (RawRem, RawProd, RawTcoc, RawSec, RawJny, RawTrnCmpSX, RawCommon, RawStop, RawOutCon, RawTrfRes, RawPrice, JnyFltr, TripSearchRequest, TvlrProf, TrfReq, RawRequestClient, RawRequestAuth, RawRequest, Cfg)

_A = TypeVar("_A")

_B = TypeVar("_B")

products : List[ProductType] = [ProductType("nationalExpress", "train", "InterCityExpress", "ICE", array_2("i", [1]), True), ProductType("national", "train", "InterCity \u0026 EuroCity", "IC/EC", array_2("i", [2]), True), ProductType("regionalExp", "train", "RegionalExpress \u0026 InterRegio", "RE/IR", array_2("i", [4]), True), ProductType("regional", "train", "Regio", "RB", array_2("i", [8]), True), ProductType("suburban", "train", "S-Bahn", "S", array_2("i", [16]), True), ProductType("bus", "bus", "Bus", "B", array_2("i", [32]), True), ProductType("ferry", "watercraft", "Ferry", "F", array_2("i", [64]), True), ProductType("subway", "train", "U-Bahn", "U", array_2("i", [128]), True), ProductType("tram", "train", "Tram", "T", array_2("i", [256]), True), ProductType("taxi", "taxi", "Group Taxi", "Taxi", array_2("i", [512]), True)]

def expr_547() -> TypeInfo:
    return record_type("FsHafas.Profiles.Db.HintByCode", [], HintByCode, lambda: [("type", string_type), ("code", string_type), ("summary", string_type)])


class HintByCode(Record):
    def __init__(self, type: str, code: str, summary: str) -> None:
        super().__init__()
        self.type = type
        self.code = code
        self.summary = summary
    

HintByCode_reflection = expr_547

hints_by_code : List[Tuple[str, HintByCode]] = [("fb", HintByCode("hint", "bicycle-conveyance", "bicycles conveyed")), ("fr", HintByCode("hint", "bicycle-conveyance-reservation", "bicycles conveyed, subject to reservation")), ("nf", HintByCode("hint", "no-bicycle-conveyance", "bicycles not conveyed")), ("k2", HintByCode("hint", "2nd-class-only", "2. class only")), ("eh", HintByCode("hint", "boarding-ramp", "vehicle-mounted boarding ramp available")), ("ro", HintByCode("hint", "wheelchairs-space", "space for wheelchairs")), ("oa", HintByCode("hint", "wheelchairs-space-reservation", "space for wheelchairs, subject to reservation")), ("wv", HintByCode("hint", "wifi", "WiFi available")), ("wi", HintByCode("hint", "wifi", "WiFi available")), ("sn", HintByCode("hint", "snacks", "snacks available for purchase")), ("mb", HintByCode("hint", "snacks", "snacks available for purchase")), ("mp", HintByCode("hint", "snacks", "snacks available for purchase at the seat")), ("bf", HintByCode("hint", "barrier-free", "barrier-free")), ("rg", HintByCode("hint", "barrier-free-vehicle", "barrier-free vehicle")), ("bt", HintByCode("hint", "on-board-bistro", "Bordbistro available")), ("br", HintByCode("hint", "on-board-restaurant", "Bordrestaurant available")), ("ki", HintByCode("hint", "childrens-area", "children\u0027s area available")), ("kk", HintByCode("hint", "parents-childrens-compartment", "parent-and-children compartment available")), ("kr", HintByCode("hint", "kids-service", "DB Kids Service available")), ("ls", HintByCode("hint", "power-sockets", "power sockets available")), ("ev", HintByCode("hint", "replacement-service", "replacement service")), ("kl", HintByCode("hint", "air-conditioned", "air-conditioned vehicle")), ("r0", HintByCode("hint", "upward-escalator", "upward escalator")), ("au", HintByCode("hint", "elevator", "elevator available")), ("ck", HintByCode("hint", "komfort-checkin", "Komfort-Checkin available")), ("it", HintByCode("hint", "ice-sprinter", "ICE Sprinter service")), ("rp", HintByCode("hint", "compulsory-reservation", "compulsory seat reservation")), ("rm", HintByCode("hint", "optional-reservation", "optional seat reservation")), ("scl", HintByCode("hint", "all-2nd-class-seats-reserved", "all 2nd class seats reserved")), ("cacl", HintByCode("hint", "all-seats-reserved", "all seats reserved")), ("sk", HintByCode("hint", "oversize-luggage-forbidden", "oversize luggage not allowed")), ("hu", HintByCode("hint", "animals-forbidden", "animals not allowed, except guide dogs")), ("ik", HintByCode("hint", "baby-cot-required", "baby cot/child seat required")), ("ee", HintByCode("hint", "on-board-entertainment", "on-board entertainment available")), ("toilet", HintByCode("hint", "toilet", "toilet available")), ("oc", HintByCode("hint", "wheelchair-accessible-toilet", "wheelchair-accessible toilet available")), ("iz", HintByCode("hint", "intercity-2", "Intercity 2"))]

codes_by_text : List[Tuple[str, str]] = [("journey cancelled", "journey-cancelled"), ("stop cancelled", "stop-cancelled"), ("signal failure", "signal-failure"), ("signalstörung", "signal-failure"), ("additional stop", "additional-stopover"), ("platform change", "changed platform")]

def parse_hint_by_code(parsed: Hint, raw: RawRem) -> Hint:
    if raw.type == "K":
        match_value : Optional[str] = raw.txt_n
        if match_value is None:
            return parsed
        
        else: 
            return Hint(parsed.type, parsed.code, parsed.summary, match_value, parsed.trip_id)
        
    
    elif raw.type == "A":
        def predicate(tupled_arg: Tuple[str, HintByCode], parsed: Hint=parsed, raw: RawRem=raw) -> bool:
            return tupled_arg[0] == raw.code.lower()
        
        match_value_1 : Optional[Tuple[str, HintByCode]] = try_find(predicate, hints_by_code)
        if match_value_1 is None:
            return parsed
        
        else: 
            h : HintByCode = match_value_1[1]
            return Hint(parsed.type, h.code, h.summary, parsed.text, parsed.trip_id)
        
    
    elif raw.txt_n is not None:
        def binder(tupled_arg_2: Tuple[str, str], parsed: Hint=parsed, raw: RawRem=raw) -> Optional[str]:
            return tupled_arg_2[1]
        
        def predicate_1(tupled_arg_1: Tuple[str, str], parsed: Hint=parsed, raw: RawRem=raw) -> bool:
            return tupled_arg_1[0] == value_2(raw.txt_n).lower()
        
        return Hint(parsed.type, bind(binder, try_find(predicate_1, codes_by_text)), parsed.summary, parsed.text, parsed.trip_id)
    
    else: 
        return parsed
    


def parse_status_by_code(parsed: Status, raw: RawRem) -> Status:
    if raw.type == "K":
        match_value : Optional[str] = raw.txt_n
        if match_value is None:
            return parsed
        
        else: 
            return Status(parsed.type, parsed.code, parsed.summary, match_value, parsed.trip_id)
        
    
    elif raw.txt_n is not None:
        def binder(tupled_arg_1: Tuple[str, str], parsed: Status=parsed, raw: RawRem=raw) -> Optional[str]:
            return tupled_arg_1[1]
        
        def predicate(tupled_arg: Tuple[str, str], parsed: Status=parsed, raw: RawRem=raw) -> bool:
            return tupled_arg[0] == value_2(raw.txt_n).lower()
        
        return Status(parsed.type, bind(binder, try_find(predicate, codes_by_text)), parsed.summary, parsed.text, parsed.trip_id)
    
    else: 
        return parsed
    


def parse_hint(parsed: Optional[Any], h: RawRem) -> Optional[Any]:
    (pattern_matching_result, parsed_hint, parsed_status) = (None, None, None)
    if parsed is not None:
        if isinstance(value_2(parsed), Hint):
            pattern_matching_result = 0
            parsed_hint = value_2(parsed)
        
        elif isinstance(value_2(parsed), Status):
            pattern_matching_result = 1
            parsed_status = value_2(parsed)
        
        else: 
            pattern_matching_result = 2
        
    
    else: 
        pattern_matching_result = 2
    
    if pattern_matching_result == 0:
        return some(parse_hint_by_code(parsed_hint, h))
    
    elif pattern_matching_result == 1:
        return some(parse_status_by_code(parsed_status, h))
    
    elif pattern_matching_result == 2:
        return parsed
    


def parse_line_with_additional_name(parsed: Line, p: RawProd) -> Line:
    if p.add_name is None:
        return parsed
    
    else: 
        return Line(parsed.type, parsed.id, p.add_name, parsed.admin_code, parsed.fahrt_nr, parsed.name, parsed.product, parsed.public, parsed.mode, parsed.routes, parsed.operator, parsed.express, parsed.metro, parsed.night, parsed.nr, parsed.symbol, parsed.directions, parsed.product_name)
    


load_factors : List[str] = ["", "low-to-medium", "high", "very-high", "exceptionally-high"]

def parse_load_factor(opt: Options, tcoc_l: List[RawTcoc], tcoc_x: MutableSequence[int]) -> Optional[str]:
    cls : str = "FIRST" if opt.first_class else "SECOND"
    def predicate(t: RawTcoc, opt: Options=opt, tcoc_l: List[RawTcoc]=tcoc_l, tcoc_x: MutableSequence[int]=tcoc_x) -> bool:
        return t.c == cls
    
    def mapping(i: int, opt: Options=opt, tcoc_l: List[RawTcoc]=tcoc_l, tcoc_x: MutableSequence[int]=tcoc_x) -> RawTcoc:
        return tcoc_l[i]
    
    match_value : Optional[RawTcoc] = try_find(predicate, map(mapping, tcoc_x, None))
    (pattern_matching_result, tcoc_1) = (None, None)
    if match_value is not None:
        if match_value.r is not None:
            pattern_matching_result = 0
            tcoc_1 = match_value
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return load_factors[value_2(tcoc_1.r)]
    
    elif pattern_matching_result == 1:
        return None
    


def parse_journey_leg_with_load_factor(parsed: Leg, ctx: Context, pt: RawSec, date: str) -> Leg:
    def arrow_548(parsed: Leg=parsed, ctx: Context=ctx, pt: RawSec=pt, date: str=date) -> Optional[MutableSequence[int]]:
        match_value : Optional[RawJny] = pt.jny
        if match_value is None:
            return None
        
        else: 
            match_value_1 : Optional[RawTrnCmpSX] = match_value.d_trn_cmp_sx
            return None if (match_value_1 is None) else match_value_1.tcoc_x
        
    
    def arrow_549(parsed: Leg=parsed, ctx: Context=ctx, pt: RawSec=pt, date: str=date) -> Optional[List[RawTcoc]]:
        match_value_2 : Optional[RawCommon] = ctx.res.common
        return None if (match_value_2 is None) else match_value_2.tcoc_l
    
    match_value_3 : Tuple[Optional[MutableSequence[int]], Optional[List[RawTcoc]]] = (arrow_548(), arrow_549())
    (pattern_matching_result, tcoc_l_1, tcoc_x_1) = (None, None, None)
    if match_value_3[0] is not None:
        if match_value_3[1] is not None:
            pattern_matching_result = 0
            tcoc_l_1 = match_value_3[1]
            tcoc_x_1 = match_value_3[0]
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return Leg(parsed.trip_id, parsed.origin, parsed.destination, parsed.departure, parsed.planned_departure, parsed.prognosed_arrival, parsed.departure_delay, parsed.departure_platform, parsed.prognosed_departure_platform, parsed.planned_departure_platform, parsed.arrival, parsed.planned_arrival, parsed.prognosed_departure, parsed.arrival_delay, parsed.arrival_platform, parsed.prognosed_arrival_platform, parsed.planned_arrival_platform, parsed.stopovers, parsed.schedule, parsed.price, parsed.operator, parsed.direction, parsed.line, parsed.reachable, parsed.cancelled, parsed.walking, parse_load_factor(ctx.opt, tcoc_l_1, tcoc_x_1), parsed.distance, parsed.public, parsed.transfer, parsed.cycle, parsed.alternatives, parsed.polyline, parsed.remarks, parsed.current_location)
    
    elif pattern_matching_result == 1:
        return parsed
    


def parse_arr_or_dep_with_load_factor(parsed: Alternative, ctx: Context, d: RawJny) -> Alternative:
    def arrow_550(parsed: Alternative=parsed, ctx: Context=ctx, d: RawJny=d) -> Optional[MutableSequence[int]]:
        match_value : Optional[RawStop] = d.stb_stop
        if match_value is None:
            return None
        
        else: 
            match_value_1 : Optional[RawTrnCmpSX] = match_value.d_trn_cmp_sx
            return None if (match_value_1 is None) else match_value_1.tcoc_x
        
    
    def arrow_551(parsed: Alternative=parsed, ctx: Context=ctx, d: RawJny=d) -> Optional[List[RawTcoc]]:
        match_value_2 : Optional[RawCommon] = ctx.res.common
        return None if (match_value_2 is None) else match_value_2.tcoc_l
    
    match_value_3 : Tuple[Optional[MutableSequence[int]], Optional[List[RawTcoc]]] = (arrow_550(), arrow_551())
    (pattern_matching_result, tcoc_l_1, tcoc_x_1) = (None, None, None)
    if match_value_3[0] is not None:
        if match_value_3[1] is not None:
            pattern_matching_result = 0
            tcoc_l_1 = match_value_3[1]
            tcoc_x_1 = match_value_3[0]
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return Alternative(parsed.trip_id, parsed.direction, parsed.location, parsed.line, parsed.stop, parsed.when, parsed.planned_when, parsed.prognosed_when, parsed.delay, parsed.platform, parsed.planned_platform, parsed.prognosed_platform, parsed.remarks, parsed.cancelled, parse_load_factor(ctx.opt, tcoc_l_1, tcoc_x_1), parsed.provenance, parsed.previous_stopovers, parsed.next_stopovers, parsed.frames, parsed.polyline, parsed.current_trip_position, parsed.origin, parsed.destination)
    
    elif pattern_matching_result == 1:
        return parsed
    


def parse_journey_with_price(parsed: Journey, raw: RawOutCon) -> Journey:
    match_value : Optional[RawTrfRes] = raw.trf_res
    (pattern_matching_result, trf_res_1) = (None, None)
    if match_value is not None:
        def arrow_553(parsed: Journey=parsed, raw: RawOutCon=raw) -> bool:
            trf_res : RawTrfRes = match_value
            return (len(trf_res.fare_set_l[0].fare_l) > 0) if (len(trf_res.fare_set_l) > 0) else False
        
        if arrow_553():
            pattern_matching_result = 0
            trf_res_1 = match_value
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        match_value_1 : Optional[RawPrice] = trf_res_1.fare_set_l[0].fare_l[0].price
        (pattern_matching_result_1, price_1) = (None, None)
        if match_value_1 is not None:
            def arrow_552(parsed: Journey=parsed, raw: RawOutCon=raw) -> bool:
                price : RawPrice = match_value_1
                return (value_2(price.amount) > 0) if (price.amount is not None) else False
            
            if arrow_552():
                pattern_matching_result_1 = 0
                price_1 = match_value_1
            
            else: 
                pattern_matching_result_1 = 1
            
        
        else: 
            pattern_matching_result_1 = 1
        
        if pattern_matching_result_1 == 0:
            return Journey(parsed.type, parsed.legs, parsed.refresh_token, parsed.remarks, Price(round(value_2(price_1.amount) / 100, 2), "EUR", None), parsed.cycle, parsed.scheduled_days)
        
        elif pattern_matching_result_1 == 1:
            return parsed
        
    
    elif pattern_matching_result == 1:
        return parsed
    


def format_station(id: str) -> str:
    if is_match(id, "^\\d{6,}$"):
        return id
    
    else: 
        raise Exception("station id: " + id)
    


bike_fltr : JnyFltr = JnyFltr("BC", "INC", None, None)

def get_option_value(opt: Optional[_A], getter: Callable[[_A], Optional[_B]], default_opt: _A) -> _B:
    default_value : _B
    match_value : Optional[_B] = getter(default_opt)
    if match_value is None:
        raise Exception("getOptionValue: value expected")
    
    else: 
        default_value = value_2(match_value)
    
    if opt is None:
        return default_value
    
    else: 
        match_value_1 : Optional[_B] = getter(value_2(opt))
        if match_value_1 is None:
            return default_value
        
        else: 
            return value_2(match_value_1)
        
    


def format_loyalty_card(data: LoyaltyCard) -> int:
    cls : int = default_arg(data.class_, 2) or 0
    match_value : Tuple[Optional[str], Optional[int]] = (data.type, data.discount)
    (pattern_matching_result, discount, type) = (None, None, None)
    if match_value[0] is not None:
        if match_value[1] is not None:
            pattern_matching_result = 0
            discount = match_value[1]
            type = match_value[0]
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        if type == "Bahncard":
            if discount == 25:
                if cls == 1:
                    return 1
                
                else: 
                    return 2
                
            
            elif discount == 50:
                if cls == 1:
                    return 3
                
                else: 
                    return 4
                
            
            else: 
                return 0
            
        
        else: 
            return 0
        
    
    elif pattern_matching_result == 1:
        return 0
    


def age_group_from_age(age: Optional[int]=None) -> str:
    if age is None:
        return "E"
    
    else: 
        age_1 : int = age or 0
        if age_1 < 6:
            return "B"
        
        elif age_1 < 15:
            return "K"
        
        elif age_1 < 27:
            return "Y"
        
        elif age_1 < 65:
            return "E"
        
        else: 
            return "S"
        
    


def transform_journeys_query(opt: Optional[JourneysOptions], q: TripSearchRequest) -> TripSearchRequest:
    def arrow_554(v: JourneysOptions, opt: Optional[JourneysOptions]=opt, q: TripSearchRequest=q) -> Optional[bool]:
        return v.bike
    
    bike : bool = get_option_value(opt, arrow_554, Default_JourneysOptions)
    def arrow_555(v_1: JourneysOptions, opt: Optional[JourneysOptions]=opt, q: TripSearchRequest=q) -> Optional[bool]:
        return v_1.first_class
    
    first_class : bool = get_option_value(opt, arrow_555, Default_JourneysOptions)
    jny_fltr_l : List[JnyFltr] = append([bike_fltr], q.jny_fltr_l, None) if bike else q.jny_fltr_l
    def arrow_556(opt: Optional[JourneysOptions]=opt, q: TripSearchRequest=q) -> Optional[int]:
        opt_2 : JourneysOptions = opt
        return format_loyalty_card(value_2(opt_2.loyalty_card))
    
    redtn_card : Optional[int] = (arrow_556() if (opt.loyalty_card is not None) else None) if (opt is not None) else None
    return TripSearchRequest(q.get_passlist, q.max_chg, q.min_chg_time, q.dep_loc_l, q.via_loc_l, q.arr_loc_l, jny_fltr_l, q.gis_fltr_l, q.get_tariff, q.ushrp, q.get_pt, q.get_iv, q.get_polyline, q.out_date, q.out_time, q.num_f, q.out_frwd, TrfReq(1 if first_class else 2, [TvlrProf(age_group_from_age(None if (opt is None) else opt.age), redtn_card)], "PK"))


req : RawRequest = RawRequest("de", [], RawRequestClient("DB", "21120000", "AND", "DB Navigator"), "DB.R21.12.a", "1.34", RawRequestAuth("AID", "n91dB8Z77MLdoR0K"))

profile : Profile = default_profile()

Profile__set__locale_Z721C83C5(profile, "de-DE")

Profile__set__timezone_Z721C83C5(profile, "Europe/Berlin")

Profile__set__endpoint_Z721C83C5(profile, "https://reiseauskunft.bahn.de/bin/mgate.exe")

Profile__set_salt_Z721C83C5(profile, "bdI8UVj40K5fvxwf")

Profile__set_cfg_Z3219B2F8(profile, Cfg("GPA", "HYBRID"))

Profile__set_baseRequest_Z42C91061(profile, req)

Profile__set__products_76A34681(profile, products)

Profile__set__trip_6FCE9E49(profile, True)

Profile__set__radar_6FCE9E49(profile, True)

Profile__set__tripsByName_6FCE9E49(profile, True)

Profile__set__reachableFrom_6FCE9E49(profile, True)

Profile__set__journeysFromTrip_6FCE9E49(profile, True)

Profile__set_journeysOutFrwd_Z1FBCCD16(profile, True)

def arrow_557(id: str) -> str:
    return format_station(id)


Profile__set_formatStation_11D407F6(profile, arrow_557)

def arrow_558(opt: Optional[JourneysOptions], q: TripSearchRequest) -> TripSearchRequest:
    return transform_journeys_query(opt, q)


Profile__set_transformJourneysQuery_4AA4AF64(profile, arrow_558)

default_parse_journey : Callable[[Context, RawOutCon], Journey] = Profile__get_parseJourney(profile)

def arrow_559(ctx: Context, p: RawOutCon) -> Journey:
    return parse_journey_with_price(default_parse_journey(ctx)(p), p)


Profile__set_parseJourney_Z1F35F4C(profile, arrow_559)

default_parse_journey_leg : Callable[[Context, RawSec, str], Leg] = Profile__get_parseJourneyLeg(profile)

def arrow_560(ctx: Context, pt: RawSec, date: str) -> Leg:
    return parse_journey_leg_with_load_factor(default_parse_journey_leg(ctx)(pt)(date), ctx, pt, date)


Profile__set_parseJourneyLeg_3913217E(profile, arrow_560)

default_parse_departure : Callable[[Context, RawJny], Alternative] = Profile__get_parseDeparture(profile)

def arrow_561(ctx: Context, pt: RawJny) -> Alternative:
    return parse_arr_or_dep_with_load_factor(default_parse_departure(ctx)(pt), ctx, pt)


Profile__set_parseDeparture_537DC5A(profile, arrow_561)

default_parse_hint : Callable[[Context, RawRem], Optional[Any]] = Profile__get_parseHint(profile)

def arrow_562(ctx: Context, p: RawRem) -> Optional[Any]:
    return parse_hint(default_parse_hint(ctx)(p), p)


Profile__set_parseHint_2044943E(profile, arrow_562)

default_parse_line : Callable[[Context, RawProd], Line] = Profile__get_parseLine(profile)

def arrow_563(ctx: Context, p: RawProd) -> Line:
    return parse_line_with_additional_name(default_parse_line(ctx)(p), p)


Profile__set_parseLine_718F82F(profile, arrow_563)




from __future__ import annotations
from hashlib import md5
from json import dumps
from requests import post
from typing import (Any, TypeVar)
from ...fable_library.async_builder import (singleton, Async)
from ...fable_library.map_util import add_to_dict
from ...fable_library.reflection import (TypeInfo, class_type)
from ...fs_hafas_python.extra_types import Log_Print

_A_ = TypeVar("_A_")

def expr_310() -> TypeInfo:
    return class_type("FsHafas.Client.Request.HttpClient", None, HttpClient)


class HttpClient:
    pass

HttpClient_reflection = expr_310

def HttpClient__ctor() -> HttpClient:
    return HttpClient()


def HttpClient__Dispose(__: HttpClient) -> None:
    pass


def HttpClient__PostAsync(__: HttpClient, url: str, salt: str, json: str) -> Async[str]:
    url_escaped : str = ((url + "?checksum=") + HttpClient__getMd5(__, json, salt)) if (len(salt) > 0) else url
    HttpClient__log(__, "url: ", url_escaped)
    headers : Any = dict([])
    add_to_dict(headers, "Content-Type", "application/json; charset=utf-8")
    add_to_dict(headers, "Accept-Encoding", "gzip, br, deflate")
    add_to_dict(headers, "Accept", "application/json")
    add_to_dict(headers, "User-Agent", "agent")
    def arrow_311(__: HttpClient=__, url: str=url, salt: str=salt, json: str=json) -> Async[str]:
        r : Any = post(url_escaped, data=json.encode('utf-8'), headers=headers)
        if (r.status_code) == 200:
            return singleton.Return(dumps(r.json()))
        
        else: 
            HttpClient__log(__, "statusCode: ", r.status_code)
            HttpClient__log(__, "text: ", r.text)
            return singleton.Return("")
        
    
    return singleton.Delay(arrow_311)


def HttpClient__log(this: HttpClient, msg: str, o: _A_) -> None:
    Log_Print(msg, o)


def HttpClient__getMd5(this: HttpClient, json: str, salt: str) -> str:
    return md5((json + salt).encode()).hexdigest()





import uuid

from .types import FSharpRef


def parse(string: str) -> uuid.UUID:
    return uuid.UUID(string)


def try_parse(string: str, def_value: FSharpRef[uuid.UUID]) -> bool:
    try:
        def_value.contents = parse(string)
        return True
    except Exception:
        return False


def to_string(guid: uuid.UUID) -> str:
    return str(guid)


def new_guid() -> uuid.UUID:
    return uuid.uuid4()



from __future__ import annotations

import array
from abc import abstractmethod
from typing import Any, Callable, Generic, Iterable, List, Optional, TypeVar
from typing import Union as Union_
from typing import cast

from .util import IComparable, compare

_T = TypeVar("_T")


class FSharpRef(Generic[_T]):
    def __init__(
        self,
        contents_or_getter: Union_[None, _T, Callable[[], _T]],
        setter: Optional[Callable[[_T], None]] = None,
    ) -> None:
        contents = cast(_T, contents_or_getter)

        def set_contents(value: _T):
            nonlocal contents
            contents = value

        if callable(setter):
            self.getter = cast(Callable[[], _T], contents_or_getter)
            self.setter = setter
        else:
            self.getter = lambda: contents
            self.setter = set_contents

    @property
    def contents(self) -> _T:
        return self.getter()

    @contents.setter
    def contents(self, v: _T) -> None:
        self.setter(v)


class Union(IComparable):
    def __init__(self):
        self.tag: int
        self.fields: List[Any] = []

    @staticmethod
    @abstractmethod
    def cases() -> List[str]:
        ...

    @property
    def name(self) -> str:
        return self.cases()[self.tag]

    def __str__(self) -> str:
        if not len(self.fields):
            return self.name

        fields = ""
        with_parens = True
        if len(self.fields) == 1:
            field = str(self.fields[0])
            with_parens = field.find(" ") >= 0
            fields = field
        else:
            fields = ", ".join(map(str, self.fields))

        return self.name + (" (" if with_parens else " ") + fields + (")" if with_parens else "")

    def __repr__(self) -> str:
        return str(self)

    def __hash__(self) -> int:
        hashes = map(hash, self.fields)
        return hash([hash(self.tag), *hashes])

    def __eq__(self, other: Any) -> bool:
        if self is other:
            return True

        if not isinstance(other, Union):
            return False

        # Different objects are not equal even with same structure
        if self.__class__ != other.__class__:
            return False

        if self.tag == other.tag:
            return self.fields == other.fields

        return False

    def __lt__(self, other: Any) -> bool:
        if self.tag == other.tag:
            return True if compare(self.fields, other.fields) < 0 else False

        return self.tag < other.tag


def record_compare_to(self: Record, other: Record) -> int:
    if self is other:
        return 0

    elif hasattr(self, "__dict__") and self.__dict__:
        for name in self.__dict__.keys():
            if self.__dict__[name] < other.__dict__.get(name):
                return -1
            elif self.__dict__[name] > other.__dict__.get(name):
                return 1

    elif hasattr(self, "__slots__") and self.__slots__:
        for name in self.__slots__:
            if getattr(self, name) < getattr(other, name):
                return -1
            elif getattr(self, name) > getattr(other, name):
                return 1

    return 0


def record_equals(self: _T, other: _T) -> bool:
    if self is other:
        return True

    if self.__class__ != other.__class__:
        return False

    if isinstance(self, Record) and isinstance(other, Record):
        return record_compare_to(self, other) == 0

    return self == other


def record_to_string(self: Record) -> str:
    if hasattr(self, "__slots__"):
        return "{ " + "\n  ".join(map(lambda slot: slot + " = " + str(getattr(self, slot)), self.__slots__)) + " }"
    else:
        return "{ " + "\n  ".join(map(lambda kv: kv[0] + " = " + str(kv[1]), self.__dict__.items())) + " }"


def record_get_hashcode(self: Record) -> int:
    slots = type(self).__slots__
    return hash(tuple(getattr(self, fixed_field) for fixed_field in slots))


class Record(IComparable):
    __slots__: List[str]

    def __str__(self) -> str:
        return record_to_string(self)

    def __repr__(self) -> str:
        return str(self)

    def GetHashCode(self) -> int:
        return record_get_hashcode(self)

    def Equals(self, other: Record) -> bool:
        return record_equals(self, other)

    def CompareTo(self, other: Record) -> int:
        return record_compare_to(self, other)

    def __lt__(self, other: Any) -> bool:
        return True if self.CompareTo(other) == -1 else False

    def __eq__(self, other: Any) -> bool:
        return self.Equals(other)

    def __hash__(self) -> int:
        return record_get_hashcode(self)


class Attribute:
    pass


def seq_to_string(self: Iterable[Any]) -> str:
    str = "["

    for count, x in enumerate(self):
        if count == 0:
            str += to_string(x)

        elif count == 100:
            str += "; ..."
            break

        else:
            str += "; " + to_string(x)

    return str + "]"


def to_string(x: Union_[Iterable[Any], Any], call_stack: int = 0) -> str:
    if x is not None:
        if isinstance(x, bool):
            return str(x).lower()

        if isinstance(x, Iterable) and not hasattr(x, "__str__"):
            return seq_to_string(x)

    return str(x)


class Exception(Exception):
    def __init__(self, msg: Optional[str] = None):
        self.msg = msg

    def __eq__(self, other: Any) -> bool:
        if self is other:
            return True

        if other is None:
            return False

        return self.msg == other.msg


class FSharpException(Exception, IComparable):
    def __init__(self):
        self.Data0: Any = None

    def __str__(self) -> str:
        return record_to_string(self)  # type: ignore

    def __repr__(self) -> str:
        return str(self)

    def __eq__(self, other: Any) -> bool:
        if self is other:
            return True

        if other is None:
            return False

        return self.Data0 == other.Data0

    def __lt__(self, other: Any) -> bool:
        if not isinstance(other, FSharpException):
            return False

        if self.Data0:
            if other.Data0:
                return self.Data0 < other.Data0
            else:
                return False

        elif not self.Data0:
            if other.Data0:
                return False
            else:
                return True

        return super().__lt__(other)

    def __hash__(self) -> int:
        return hash(self.Data0)

    def GetHashCode(self) -> int:
        return hash(self)

    def Equals(self, other: FSharpException):
        return record_equals(self, other)

    def CompareTo(self, other: FSharpException):
        return compare(self.Data0, other.Data0)


def Int8Array(lst: List[int]):
    return array.array("b", lst)


def Uint8Array(lst: List[int]):
    return bytearray(lst)


def Int16Array(lst: List[int]):
    return array.array("h", lst)


def Uint16Array(lst: List[int]):
    return array.array("H", lst)


def Int32Array(lst: List[int]):
    return array.array("i", lst)


def Uint32Array(lst: List[int]):
    return array.array("I", lst)


def Float32Array(lst: List[float]):
    return array.array("f", lst)


def Float64Array(lst: List[float]):
    return array.array("d", lst)


def is_exception(x: Any):
    return isinstance(x, Exception)


__all__ = [
    "Attribute",
    "Exception",
    "is_exception",
    "Int8Array",
    "Uint8Array",
    "Int16Array",
    "Uint16Array",
    "Int32Array",
    "Uint32Array",
    "Float32Array",
    "Float64Array",
    "FSharpException",
    "FSharpRef",
    "Record",
    "seq_to_string",
    "to_string",
    "Union",
]



from __future__ import annotations
from typing import (List, Any, Optional, TypeVar, Callable, MutableSequence, Tuple)
from ...fable_library.array import (map, choose, append)
from ...fable_library.option import (default_arg, some)
from ...fs_hafas_python.context import (Context, CommonData, Profile__get_parseOperator, Profile__get_parseLine, Profile__get_parseHint, Profile__get_parseIcon, Profile__get_parsePolyline, Profile__get_parseLocations)
from ...fs_hafas_python.extra_types import Icon
from ...fs_hafas_python.types_hafas_client import (Operator, Line, FeatureCollection)
from ...fs_hafas_python.types_raw_hafas_client import (RawCommon, RawOp, RawProd, RawRem, RawIco, RawPoly, RawMsg)

_A = TypeVar("_A")

_B = TypeVar("_B")

def update_operators(ctx: Context, ops: List[Operator]) -> Context:
    def arrow_358(ctx: Context=ctx, ops: List[Operator]=ops) -> CommonData:
        input_record : CommonData = ctx.common
        return CommonData(ops, input_record.locations, input_record.lines, input_record.hints, input_record.icons, input_record.polylines)
    
    return Context(ctx.profile, ctx.opt, arrow_358(), ctx.res)


def update_lines(ctx: Context, lines: List[Line]) -> Context:
    def arrow_359(ctx: Context=ctx, lines: List[Line]=lines) -> CommonData:
        input_record : CommonData = ctx.common
        return CommonData(input_record.operators, input_record.locations, lines, input_record.hints, input_record.icons, input_record.polylines)
    
    return Context(ctx.profile, ctx.opt, arrow_359(), ctx.res)


def parse_common(ctx: Context, c: RawCommon) -> CommonData:
    def mapping(op: RawOp, ctx: Context=ctx, c: RawCommon=c) -> Operator:
        return Profile__get_parseOperator(ctx.profile)(ctx)(op)
    
    ctx1 : Context = update_operators(ctx, map(mapping, default_arg(c.op_l, []), None))
    def mapping_1(p: RawProd, ctx: Context=ctx, c: RawCommon=c) -> Line:
        return Profile__get_parseLine(ctx1.profile)(ctx1)(p)
    
    ctx2 : Context = update_lines(ctx, map(mapping_1, default_arg(c.prod_l, []), None))
    def mapping_2(p_1: RawRem, ctx: Context=ctx, c: RawCommon=c) -> Optional[Any]:
        return Profile__get_parseHint(ctx2.profile)(ctx2)(p_1)
    
    hints : List[Optional[Any]] = map(mapping_2, default_arg(c.rem_l, []), None)
    def chooser(x: Optional[Icon]=None, ctx: Context=ctx, c: RawCommon=c) -> Optional[Icon]:
        return x
    
    def mapping_3(i: RawIco, ctx: Context=ctx, c: RawCommon=c) -> Optional[Icon]:
        return Profile__get_parseIcon(ctx2.profile)(ctx2)(i)
    
    icons : List[Icon] = choose(chooser, map(mapping_3, default_arg(c.ico_l, []), None), None)
    def mapping_4(p_2: RawPoly, ctx: Context=ctx, c: RawCommon=c) -> FeatureCollection:
        return Profile__get_parsePolyline(ctx2.profile)(ctx2)(p_2)
    
    polylines : List[FeatureCollection] = map(mapping_4, default_arg(c.poly_l, []), None)
    return CommonData(ctx2.common.operators, Profile__get_parseLocations(ctx2.profile)(ctx2)(default_arg(c.loc_l, [])), ctx2.common.lines, hints, icons, polylines)


def get_element_at(index: int, arr: List[_A]) -> Optional[_A]:
    if index < len(arr):
        return some(arr[index])
    
    else: 
        return None
    


def get_element_at_some(index: Optional[int], arr: List[_A]) -> Optional[_A]:
    (pattern_matching_result, index_2) = (None, None)
    if index is not None:
        if index < len(arr):
            pattern_matching_result = 0
            index_2 = index
        
        else: 
            pattern_matching_result = 1
        
    
    else: 
        pattern_matching_result = 1
    
    if pattern_matching_result == 0:
        return some(arr[index_2])
    
    elif pattern_matching_result == 1:
        return None
    


def get_array(common: Optional[RawCommon], getter: Callable[[RawCommon], Optional[List[_A]]]) -> List[_A]:
    if common is None:
        return []
    
    else: 
        match_value : Optional[List[_A]] = getter(common)
        if match_value is None:
            return []
        
        else: 
            return match_value
        
    


def map_index_array(common: Optional[RawCommon], get_target_array: Callable[[RawCommon], Optional[List[_A]]], index_arr: Optional[MutableSequence[int]]=None) -> List[_A]:
    elements : List[_A] = get_array(common, get_target_array)
    if index_arr is None:
        return []
    
    else: 
        def chooser(x: Optional[_A]=None, common: Optional[RawCommon]=common, get_target_array: Callable[[RawCommon], Optional[List[_A]]]=get_target_array, index_arr: Optional[MutableSequence[int]]=index_arr) -> Optional[_A]:
            return x
        
        def mapping(i: int, common: Optional[RawCommon]=common, get_target_array: Callable[[RawCommon], Optional[List[_A]]]=get_target_array, index_arr: Optional[MutableSequence[int]]=index_arr) -> Optional[_A]:
            return get_element_at(i, elements)
        
        return choose(chooser, map(mapping, index_arr, None), None)
    


def append_some_array(arr1: Optional[List[_A]]=None, arr2: Optional[List[_A]]=None) -> Optional[List[_A]]:
    match_value : Tuple[Optional[List[_A]], Optional[List[_A]]] = (arr1, arr2)
    if match_value[0] is None:
        if match_value[1] is not None:
            arr2_2 : List[_A] = match_value[1]
            return arr2_2
        
        else: 
            return None
        
    
    elif match_value[1] is None:
        arr1_2 : List[_A] = match_value[0]
        return arr1_2
    
    else: 
        arr1_1 : List[_A] = match_value[0]
        arr2_1 : List[_A] = match_value[1]
        return append(arr1_1, arr2_1, None)
    


def map_array(target_array: List[_B], get_index: Callable[[_A], Optional[int]], source_array: Optional[List[_A]]=None) -> List[_B]:
    if source_array is None:
        return []
    
    else: 
        def chooser(x: Optional[_B]=None, target_array: List[_B]=target_array, get_index: Callable[[_A], Optional[int]]=get_index, source_array: Optional[List[_A]]=source_array) -> Optional[_B]:
            return x
        
        def mapping(elt: Optional[_A]=None, target_array: List[_B]=target_array, get_index: Callable[[_A], Optional[int]]=get_index, source_array: Optional[List[_A]]=source_array) -> Optional[_B]:
            return get_element_at_some(get_index(elt), target_array)
        
        return choose(chooser, map(mapping, source_array, None), None)
    


def to_option(arr: List[_A]) -> Optional[List[_A]]:
    if len(arr) > 0:
        return arr
    
    else: 
        return None
    


def msg_lto_remarks(ctx: Context, msg_l: Optional[List[RawMsg]]=None) -> Optional[List[Any]]:
    def chooser(x_1: Optional[Any]=None, ctx: Context=ctx, msg_l: Optional[List[RawMsg]]=msg_l) -> Optional[Any]:
        return x_1
    
    def get_index(x: RawMsg, ctx: Context=ctx, msg_l: Optional[List[RawMsg]]=msg_l) -> Optional[int]:
        return x.rem_x
    
    return to_option(choose(chooser, map_array(ctx.common.hints, get_index, msg_l), None))





from abc import abstractmethod
from typing import (MutableSequence, Protocol)

class IExports(Protocol):
    @abstractmethod
    def acos(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def asin(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def atan(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def atan2(self, __arg0: float, __arg1: float) -> float:
        ...
    
    @abstractmethod
    def comb(self, __arg0: int, __arg1: int) -> int:
        ...
    
    @abstractmethod
    def copysign(self, __arg0: float, __arg1: int) -> float:
        ...
    
    @abstractmethod
    def cos(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def dist(self, __arg0: float, __arg1: float) -> float:
        ...
    
    @abstractmethod
    def exp(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def expm1(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def fabs(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def factorial(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def fmod(self, __arg0: int, __arg1: int) -> int:
        ...
    
    @abstractmethod
    def gcd(self, ints: MutableSequence[int]) -> int:
        ...
    
    @abstractmethod
    def lcm(self, ints: MutableSequence[int]) -> int:
        ...
    
    @abstractmethod
    def log(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def log10(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def log1p(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def log2(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def pow(self, __arg0: float, __arg1: float) -> float:
        ...
    
    @abstractmethod
    def sin(self, __arg0: float) -> float:
        ...
    
    @abstractmethod
    def tan(self, __arg0: float) -> float:
        ...
    




from __future__ import annotations
from typing import (Any, Optional, Tuple, List, TypeVar, Callable)
from ..fable_library.array import (map, for_all)
from ..fable_library.mutable_map import Dictionary
from ..fable_library.reflection import (full_name, is_record, name, get_record_elements, get_generics, is_union, get_union_case_fields, get_union_cases, is_function, get_function_elements, is_array, get_element_type, is_tuple, get_tuple_elements, is_enum, get_enum_underlying_type, equals)
from ..fable_library.seq import (to_array, delay, append, singleton, collect)
from ..fable_library.string import trim_start
from ..fable_library.util import (IEnumerable, Lazy, structural_hash)
from .type_info import (TypeInfo, RecordField, UnionCase)

_A_ = TypeVar("_A_")

def _007CPrimitiveType_007C__007C(prim_type: Any) -> Optional[TypeInfo]:
    match_value : str = full_name(prim_type)
    (pattern_matching_result,) = (None,)
    if match_value == "System.String":
        pattern_matching_result = 0
    
    elif match_value == "System.Char":
        pattern_matching_result = 1
    
    elif match_value == "System.Int16":
        pattern_matching_result = 2
    
    elif match_value == "System.Int32":
        pattern_matching_result = 3
    
    elif match_value == "Microsoft.FSharp.Core.int64`1":
        pattern_matching_result = 4
    
    elif match_value == "System.Int64":
        pattern_matching_result = 4
    
    elif match_value == "System.UInt16":
        pattern_matching_result = 5
    
    elif match_value == "System.UInt32":
        pattern_matching_result = 6
    
    elif match_value == "System.UInt64":
        pattern_matching_result = 7
    
    elif match_value == "System.DateTime":
        pattern_matching_result = 8
    
    elif match_value == "System.TimeSpan":
        pattern_matching_result = 9
    
    elif match_value == "System.DateTimeOffset":
        pattern_matching_result = 10
    
    elif match_value == "System.Boolean":
        pattern_matching_result = 11
    
    elif match_value == "System.Single":
        pattern_matching_result = 12
    
    elif match_value == "System.Double":
        pattern_matching_result = 13
    
    elif match_value == "Microsoft.FSharp.Core.decimal`1":
        pattern_matching_result = 14
    
    elif match_value == "System.Decimal":
        pattern_matching_result = 14
    
    elif match_value == "System.Numerics.BigInteger":
        pattern_matching_result = 15
    
    elif match_value == "Microsoft.FSharp.Core.Unit":
        pattern_matching_result = 16
    
    elif match_value == "System.Guid":
        pattern_matching_result = 17
    
    elif match_value == "System.Byte":
        pattern_matching_result = 18
    
    elif match_value == "System.SByte":
        pattern_matching_result = 19
    
    elif match_value == "System.Object":
        pattern_matching_result = 20
    
    elif match_value == "System.Uri":
        pattern_matching_result = 21
    
    else: 
        pattern_matching_result = 22
    
    if pattern_matching_result == 0:
        return TypeInfo(2)
    
    elif pattern_matching_result == 1:
        return TypeInfo(1)
    
    elif pattern_matching_result == 2:
        return TypeInfo(11)
    
    elif pattern_matching_result == 3:
        return TypeInfo(6)
    
    elif pattern_matching_result == 4:
        return TypeInfo(12)
    
    elif pattern_matching_result == 5:
        return TypeInfo(3)
    
    elif pattern_matching_result == 6:
        return TypeInfo(4)
    
    elif pattern_matching_result == 7:
        return TypeInfo(5)
    
    elif pattern_matching_result == 8:
        return TypeInfo(15)
    
    elif pattern_matching_result == 9:
        return TypeInfo(18)
    
    elif pattern_matching_result == 10:
        return TypeInfo(16)
    
    elif pattern_matching_result == 11:
        return TypeInfo(7)
    
    elif pattern_matching_result == 12:
        return TypeInfo(8)
    
    elif pattern_matching_result == 13:
        return TypeInfo(9)
    
    elif pattern_matching_result == 14:
        return TypeInfo(10)
    
    elif pattern_matching_result == 15:
        return TypeInfo(17)
    
    elif pattern_matching_result == 16:
        return TypeInfo(0)
    
    elif pattern_matching_result == 17:
        return TypeInfo(19)
    
    elif pattern_matching_result == 18:
        return TypeInfo(13)
    
    elif pattern_matching_result == 19:
        return TypeInfo(14)
    
    elif pattern_matching_result == 20:
        return TypeInfo(20)
    
    elif pattern_matching_result == 21:
        return TypeInfo(21)
    
    elif pattern_matching_result == 22:
        return None
    


def _007CRecordType_007C__007C(t: Any) -> Optional[List[Tuple[Any, str, Any]]]:
    if is_record(t):
        def mapping(field: Any, t: Any=t) -> Tuple[Any, str, Any]:
            return (field, name(field), field[1])
        
        return map(mapping, get_record_elements(t), None)
    
    else: 
        return None
    


def _007CSetType_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("Microsoft.FSharp.Collections.FSharpSet`1") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def _007CNullable_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("System.Nullable`1") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def _007CUnionType_007C__007C(t: Any) -> Optional[List[Tuple[str, Any, List[Any]]]]:
    if is_union(t):
        def mapping_1(info: Any, t: Any=t) -> Tuple[str, Any, List[Any]]:
            def mapping(prop: Any, info: Any=info) -> Any:
                return prop[1]
            
            return (name(info), info, map(mapping, get_union_case_fields(info), None))
        
        return map(mapping_1, get_union_cases(t), None)
    
    else: 
        return None
    


def _007CMapType_007C__007C(t: Any) -> Optional[Tuple[Any, Any]]:
    if trim_start(full_name(t), "$").find("Microsoft.FSharp.Collections.FSharpMap`2") == 0:
        gen_args : List[Any] = get_generics(t)
        return (gen_args[0], gen_args[1])
    
    else: 
        return None
    


def _007CListType_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("Microsoft.FSharp.Collections.FSharpList`1") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def flatten_func_types(type_def: Any) -> List[Any]:
    def arrow_8(type_def: Any=type_def) -> IEnumerable[Any]:
        if is_function(type_def):
            pattern_input : Tuple[Any, Any] = get_function_elements(type_def)
            def arrow_7(__unit: Any=None) -> IEnumerable[Any]:
                return flatten_func_types(pattern_input[1])
            
            return append(flatten_func_types(pattern_input[0]), delay(arrow_7))
        
        else: 
            return singleton(type_def)
        
    
    return to_array(delay(arrow_8))


def _007CFuncType_007C__007C(t: Any) -> Optional[List[Any]]:
    if is_function(t):
        return flatten_func_types(t)
    
    else: 
        return None
    


def _007CArrayType_007C__007C(t: Any) -> Optional[Any]:
    if is_array(t):
        return get_element_type(t)
    
    else: 
        return None
    


def _007COptionType_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("Microsoft.FSharp.Core.FSharpOption`1") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def _007CTupleType_007C__007C(t: Any) -> Optional[List[Any]]:
    if is_tuple(t):
        return get_tuple_elements(t)
    
    else: 
        return None
    


def _007CSeqType_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("System.Collections.Generic.IEnumerable`1") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def _007CDictionaryType_007C__007C(t: Any) -> Optional[Tuple[Any, Any]]:
    if trim_start(full_name(t), "$").find("System.Collections.Generic.Dictionary") == 0:
        gen_args : List[Any] = get_generics(t)
        return (gen_args[0], gen_args[1])
    
    else: 
        return None
    


def _007CResizeArrayType_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("System.Collections.Generic.List") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def _007CHashSetType_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("System.Collections.Generic.HashSet") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def _007CAsyncType_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("Microsoft.FSharp.Control.FSharpAsync`1") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def _007CPromiseType_007C__007C(t: Any) -> Optional[Any]:
    if trim_start(full_name(t), "$").find("Fable.Core.JS.Promise`1") == 0:
        return get_generics(t)[0]
    
    else: 
        return None
    


def lazy_to_delayed(l: Any, unit_var0: None) -> _A_:
    return l.Value


def _007CEnumType_007C__007C(t: Any) -> Optional[Any]:
    if is_enum(t):
        return get_enum_underlying_type(t)
    
    else: 
        return None
    


def _createTypeInfo(resolved_type: Any) -> TypeInfo:
    active_pattern_result571 : Optional[TypeInfo] = _007CPrimitiveType_007C__007C(resolved_type)
    if active_pattern_result571 is not None:
        type_info : TypeInfo = active_pattern_result571
        return type_info
    
    else: 
        active_pattern_result570 : Optional[List[Any]] = _007CFuncType_007C__007C(resolved_type)
        if active_pattern_result570 is not None:
            types : List[Any] = active_pattern_result570
            def arrow_14(resolved_type: Any=resolved_type) -> Callable[[], List[TypeInfo]]:
                def arrow_12(__unit: Any=None) -> List[TypeInfo]:
                    def arrow_11(resolved_type_1: Any) -> TypeInfo:
                        return create_type_info(resolved_type_1)
                    
                    return map(arrow_11, types, None)
                
                l : Any = Lazy(arrow_12)
                def arrow_13(__unit: Any=None) -> List[TypeInfo]:
                    return lazy_to_delayed(l, None)
                
                return arrow_13
            
            return TypeInfo(35, arrow_14())
        
        else: 
            active_pattern_result569 : Optional[List[Tuple[Any, str, Any]]] = _007CRecordType_007C__007C(resolved_type)
            if active_pattern_result569 is not None:
                fields : List[Tuple[Any, str, Any]] = active_pattern_result569
                def arrow_17(resolved_type: Any=resolved_type) -> Tuple[List[RecordField], Any]:
                    def arrow_16(__unit: Any=None) -> IEnumerable[RecordField]:
                        def arrow_15(match_value: Tuple[Any, str, Any]) -> IEnumerable[RecordField]:
                            return singleton(RecordField(match_value[1], create_type_info(match_value[2]), match_value[0]))
                        
                        return collect(arrow_15, fields)
                    
                    return (to_array(delay(arrow_16)), resolved_type)
                
                return TypeInfo(37, arrow_17)
            
            else: 
                active_pattern_result568 : Optional[List[Tuple[str, Any, List[Any]]]] = _007CUnionType_007C__007C(resolved_type)
                if active_pattern_result568 is not None:
                    cases : List[Tuple[str, Any, List[Any]]] = active_pattern_result568
                    def arrow_21(resolved_type: Any=resolved_type) -> Tuple[List[UnionCase], Any]:
                        def arrow_20(__unit: Any=None) -> IEnumerable[UnionCase]:
                            def arrow_19(match_value_1: Tuple[str, Any, List[Any]]) -> IEnumerable[UnionCase]:
                                def arrow_18(resolved_type_2: Any) -> TypeInfo:
                                    return create_type_info(resolved_type_2)
                                
                                return singleton(UnionCase(match_value_1[0], map(arrow_18, match_value_1[2], None), match_value_1[1]))
                            
                            return collect(arrow_19, cases)
                        
                        return (to_array(delay(arrow_20)), resolved_type)
                    
                    l_1 : Any = Lazy(arrow_21)
                    def arrow_22(resolved_type: Any=resolved_type) -> Tuple[List[UnionCase], Any]:
                        return lazy_to_delayed(l_1, None)
                    
                    return TypeInfo(38, arrow_22)
                
                else: 
                    active_pattern_result567 : Optional[Any] = _007CEnumType_007C__007C(resolved_type)
                    if active_pattern_result567 is not None:
                        elem_type : Any = active_pattern_result567
                        def arrow_25(resolved_type: Any=resolved_type) -> Callable[[], Tuple[TypeInfo, Any]]:
                            def arrow_23(__unit: Any=None) -> Tuple[TypeInfo, Any]:
                                return (create_type_info(elem_type), resolved_type)
                            
                            l_2 : Any = Lazy(arrow_23)
                            def arrow_24(__unit: Any=None) -> Tuple[TypeInfo, Any]:
                                return lazy_to_delayed(l_2, None)
                            
                            return arrow_24
                        
                        return TypeInfo(36, arrow_25())
                    
                    else: 
                        active_pattern_result566 : Optional[Any] = _007CListType_007C__007C(resolved_type)
                        if active_pattern_result566 is not None:
                            elem_type_1 : Any = active_pattern_result566
                            def arrow_26(resolved_type: Any=resolved_type) -> TypeInfo:
                                return create_type_info(elem_type_1)
                            
                            return TypeInfo(26, arrow_26)
                        
                        else: 
                            active_pattern_result565 : Optional[Any] = _007CResizeArrayType_007C__007C(resolved_type)
                            if active_pattern_result565 is not None:
                                elem_type_2 : Any = active_pattern_result565
                                def arrow_29(resolved_type: Any=resolved_type) -> Callable[[], TypeInfo]:
                                    def arrow_27(__unit: Any=None) -> TypeInfo:
                                        return create_type_info(elem_type_2)
                                    
                                    l_3 : Any = Lazy(arrow_27)
                                    def arrow_28(__unit: Any=None) -> TypeInfo:
                                        return lazy_to_delayed(l_3, None)
                                    
                                    return arrow_28
                                
                                return TypeInfo(33, arrow_29())
                            
                            else: 
                                active_pattern_result564 : Optional[Any] = _007CHashSetType_007C__007C(resolved_type)
                                if active_pattern_result564 is not None:
                                    elem_type_3 : Any = active_pattern_result564
                                    def arrow_32(resolved_type: Any=resolved_type) -> Callable[[], TypeInfo]:
                                        def arrow_30(__unit: Any=None) -> TypeInfo:
                                            return create_type_info(elem_type_3)
                                        
                                        l_4 : Any = Lazy(arrow_30)
                                        def arrow_31(__unit: Any=None) -> TypeInfo:
                                            return lazy_to_delayed(l_4, None)
                                        
                                        return arrow_31
                                    
                                    return TypeInfo(34, arrow_32())
                                
                                else: 
                                    active_pattern_result563 : Optional[Any] = _007CArrayType_007C__007C(resolved_type)
                                    if active_pattern_result563 is not None:
                                        elem_type_4 : Any = active_pattern_result563
                                        def arrow_35(resolved_type: Any=resolved_type) -> Callable[[], TypeInfo]:
                                            def arrow_33(__unit: Any=None) -> TypeInfo:
                                                return create_type_info(elem_type_4)
                                            
                                            l_5 : Any = Lazy(arrow_33)
                                            def arrow_34(__unit: Any=None) -> TypeInfo:
                                                return lazy_to_delayed(l_5, None)
                                            
                                            return arrow_34
                                        
                                        return TypeInfo(28, arrow_35())
                                    
                                    else: 
                                        active_pattern_result562 : Optional[List[Any]] = _007CTupleType_007C__007C(resolved_type)
                                        if active_pattern_result562 is not None:
                                            types_1 : List[Any] = active_pattern_result562
                                            def arrow_39(resolved_type: Any=resolved_type) -> Callable[[], List[TypeInfo]]:
                                                def arrow_37(__unit: Any=None) -> List[TypeInfo]:
                                                    def arrow_36(resolved_type_3: Any) -> TypeInfo:
                                                        return create_type_info(resolved_type_3)
                                                    
                                                    return map(arrow_36, types_1, None)
                                                
                                                l_6 : Any = Lazy(arrow_37)
                                                def arrow_38(__unit: Any=None) -> List[TypeInfo]:
                                                    return lazy_to_delayed(l_6, None)
                                                
                                                return arrow_38
                                            
                                            return TypeInfo(30, arrow_39())
                                        
                                        else: 
                                            active_pattern_result561 : Optional[Any] = _007COptionType_007C__007C(resolved_type)
                                            if active_pattern_result561 is not None:
                                                elem_type_5 : Any = active_pattern_result561
                                                def arrow_40(resolved_type: Any=resolved_type) -> TypeInfo:
                                                    return create_type_info(elem_type_5)
                                                
                                                return TypeInfo(25, arrow_40)
                                            
                                            else: 
                                                active_pattern_result560 : Optional[Any] = _007CNullable_007C__007C(resolved_type)
                                                if active_pattern_result560 is not None:
                                                    elem_type_6 : Any = active_pattern_result560
                                                    def arrow_43(resolved_type: Any=resolved_type) -> Callable[[], TypeInfo]:
                                                        def arrow_41(__unit: Any=None) -> TypeInfo:
                                                            return create_type_info(elem_type_6)
                                                        
                                                        l_7 : Any = Lazy(arrow_41)
                                                        def arrow_42(__unit: Any=None) -> TypeInfo:
                                                            return lazy_to_delayed(l_7, None)
                                                        
                                                        return arrow_42
                                                    
                                                    return TypeInfo(25, arrow_43())
                                                
                                                else: 
                                                    active_pattern_result559 : Optional[Any] = _007CSetType_007C__007C(resolved_type)
                                                    if active_pattern_result559 is not None:
                                                        elem_type_7 : Any = active_pattern_result559
                                                        def arrow_46(resolved_type: Any=resolved_type) -> Callable[[], TypeInfo]:
                                                            def arrow_44(__unit: Any=None) -> TypeInfo:
                                                                return create_type_info(elem_type_7)
                                                            
                                                            l_8 : Any = Lazy(arrow_44)
                                                            def arrow_45(__unit: Any=None) -> TypeInfo:
                                                                return lazy_to_delayed(l_8, None)
                                                            
                                                            return arrow_45
                                                        
                                                        return TypeInfo(27, arrow_46())
                                                    
                                                    else: 
                                                        active_pattern_result558 : Optional[Tuple[Any, Any]] = _007CMapType_007C__007C(resolved_type)
                                                        if active_pattern_result558 is not None:
                                                            key_type : Any = active_pattern_result558[0]
                                                            value_type : Any = active_pattern_result558[1]
                                                            def arrow_51(resolved_type: Any=resolved_type) -> Callable[[], Tuple[TypeInfo, TypeInfo]]:
                                                                def arrow_47(__unit: Any=None) -> Tuple[TypeInfo, TypeInfo]:
                                                                    return (create_type_info(key_type), create_type_info(value_type))
                                                                
                                                                l_9 : Any = Lazy(arrow_47)
                                                                def arrow_50(__unit: Any=None) -> Tuple[TypeInfo, TypeInfo]:
                                                                    return lazy_to_delayed(l_9, None)
                                                                
                                                                return arrow_50
                                                            
                                                            return TypeInfo(31, arrow_51())
                                                        
                                                        else: 
                                                            active_pattern_result557 : Optional[Tuple[Any, Any]] = _007CDictionaryType_007C__007C(resolved_type)
                                                            if active_pattern_result557 is not None:
                                                                key_type_1 : Any = active_pattern_result557[0]
                                                                value_type_1 : Any = active_pattern_result557[1]
                                                                def arrow_54(resolved_type: Any=resolved_type) -> Callable[[], Tuple[TypeInfo, TypeInfo, Any]]:
                                                                    def arrow_52(__unit: Any=None) -> Tuple[TypeInfo, TypeInfo, Any]:
                                                                        return (create_type_info(key_type_1), create_type_info(value_type_1), value_type_1)
                                                                    
                                                                    l_10 : Any = Lazy(arrow_52)
                                                                    def arrow_53(__unit: Any=None) -> Tuple[TypeInfo, TypeInfo, Any]:
                                                                        return lazy_to_delayed(l_10, None)
                                                                    
                                                                    return arrow_53
                                                                
                                                                return TypeInfo(32, arrow_54())
                                                            
                                                            else: 
                                                                active_pattern_result556 : Optional[Any] = _007CSeqType_007C__007C(resolved_type)
                                                                if active_pattern_result556 is not None:
                                                                    elem_type_8 : Any = active_pattern_result556
                                                                    def arrow_57(resolved_type: Any=resolved_type) -> Callable[[], TypeInfo]:
                                                                        def arrow_55(__unit: Any=None) -> TypeInfo:
                                                                            return create_type_info(elem_type_8)
                                                                        
                                                                        l_11 : Any = Lazy(arrow_55)
                                                                        def arrow_56(__unit: Any=None) -> TypeInfo:
                                                                            return lazy_to_delayed(l_11, None)
                                                                        
                                                                        return arrow_56
                                                                    
                                                                    return TypeInfo(29, arrow_57())
                                                                
                                                                else: 
                                                                    active_pattern_result555 : Optional[Any] = _007CAsyncType_007C__007C(resolved_type)
                                                                    if active_pattern_result555 is not None:
                                                                        elem_type_9 : Any = active_pattern_result555
                                                                        def arrow_60(resolved_type: Any=resolved_type) -> Callable[[], TypeInfo]:
                                                                            def arrow_58(__unit: Any=None) -> TypeInfo:
                                                                                return create_type_info(elem_type_9)
                                                                            
                                                                            l_12 : Any = Lazy(arrow_58)
                                                                            def arrow_59(__unit: Any=None) -> TypeInfo:
                                                                                return lazy_to_delayed(l_12, None)
                                                                            
                                                                            return arrow_59
                                                                        
                                                                        return TypeInfo(23, arrow_60())
                                                                    
                                                                    else: 
                                                                        active_pattern_result554 : Optional[Any] = _007CPromiseType_007C__007C(resolved_type)
                                                                        if active_pattern_result554 is not None:
                                                                            elem_type_10 : Any = active_pattern_result554
                                                                            def arrow_63(resolved_type: Any=resolved_type) -> Callable[[], TypeInfo]:
                                                                                def arrow_61(__unit: Any=None) -> TypeInfo:
                                                                                    return create_type_info(elem_type_10)
                                                                                
                                                                                l_13 : Any = Lazy(arrow_61)
                                                                                def arrow_62(__unit: Any=None) -> TypeInfo:
                                                                                    return lazy_to_delayed(l_13, None)
                                                                                
                                                                                return arrow_62
                                                                            
                                                                            return TypeInfo(24, arrow_63())
                                                                        
                                                                        else: 
                                                                            def arrow_66(resolved_type: Any=resolved_type) -> Callable[[], Any]:
                                                                                def arrow_64(__unit: Any=None) -> Any:
                                                                                    return resolved_type
                                                                                
                                                                                l_14 : Any = Lazy(arrow_64)
                                                                                def arrow_65(__unit: Any=None) -> Any:
                                                                                    return lazy_to_delayed(l_14, None)
                                                                                
                                                                                return arrow_65
                                                                            
                                                                            return TypeInfo(22, arrow_66())
                                                                        
                                                                    
                                                                
                                                            
                                                        
                                                    
                                                
                                            
                                        
                                    
                                
                            
                        
                    
                
            
        
    


class ObjectExpr69:
    @property
    def Equals(self) -> Any:
        def arrow_67(x: Any, y: Any) -> bool:
            return equals(x, y)
        
        return arrow_67
    
    @property
    def GetHashCode(self) -> Any:
        def arrow_68(x: Any) -> int:
            return structural_hash(x)
        
        return arrow_68
    

type_info_cache : Any = Dictionary([], ObjectExpr69())

def create_type_info(resolved_type: Any) -> TypeInfo:
    return _createTypeInfo(resolved_type)


def is_primitive(_arg1: TypeInfo) -> bool:
    if (((((((((((((((((_arg1.tag == 0) or (_arg1.tag == 2)) or (_arg1.tag == 3)) or (_arg1.tag == 4)) or (_arg1.tag == 5)) or (_arg1.tag == 6)) or (_arg1.tag == 7)) or (_arg1.tag == 8)) or (_arg1.tag == 9)) or (_arg1.tag == 10)) or (_arg1.tag == 11)) or (_arg1.tag == 12)) or (_arg1.tag == 13)) or (_arg1.tag == 15)) or (_arg1.tag == 16)) or (_arg1.tag == 17)) or (_arg1.tag == 19)) or (_arg1.tag == 25):
        return True
    
    else: 
        return False
    


def enum_union(_arg1: TypeInfo) -> bool:
    if _arg1.tag == 38:
        def predicate(case: UnionCase, _arg1: TypeInfo=_arg1) -> bool:
            return len(case.CaseTypes) == 0
        
        return for_all(predicate, _arg1.fields[0]()[0])
    
    else: 
        return False
    





from __future__ import annotations
from typing import (Optional, Any, Tuple, List)
from ...fable_library.array import (choose, map_indexed)
from ...fable_library.option import value as value_1
from ...fs_hafas_python.context import (Context, Profile__get_parseStopovers, Profile__get_parsePolyline)
from ...fs_hafas_python.lib.transformations import (Default_Location, Coordinate_toFloat, U2StopLocation_FromSomeU3StationStopLocation)
from ...fs_hafas_python.types_hafas_client import (Location, Frame, FeatureCollection, Movement)
from ...fs_hafas_python.types_raw_hafas_client import (RawJny, RawCrd, RawAni, RawPoly, PolyG)
from .common import get_element_at

def parse_movement(ctx: Context, m: RawJny) -> Movement:
    def arrow_425(ctx: Context=ctx, m: RawJny=m) -> Optional[Location]:
        match_value : Optional[RawCrd] = m.pos
        if match_value is None:
            return None
        
        else: 
            pos : RawCrd = match_value
            return Location(Default_Location.type, Default_Location.id, Default_Location.name, Default_Location.poi, Default_Location.address, Coordinate_toFloat(pos.x), Coordinate_toFloat(pos.y), Default_Location.altitude, Default_Location.distance)
        
    
    def arrow_426(ctx: Context=ctx, m: RawJny=m) -> Optional[List[Frame]]:
        match_value_1 : Optional[RawAni] = m.ani
        if match_value_1 is None:
            return None
        
        else: 
            ani : RawAni = match_value_1
            def chooser(x: Optional[Frame]=None) -> Optional[Frame]:
                return x
            
            def mapping(i: int, ms: int) -> Optional[Frame]:
                match_value_2 : Tuple[Optional[Any], Optional[Any]] = (U2StopLocation_FromSomeU3StationStopLocation(get_element_at(ani.f_loc_x[i], ctx.common.locations)), U2StopLocation_FromSomeU3StationStopLocation(get_element_at(ani.t_loc_x[i], ctx.common.locations)))
                (pattern_matching_result, destination_1, origin_1) = (None, None, None)
                if match_value_2[0] is not None:
                    if match_value_2[1] is not None:
                        pattern_matching_result = 0
                        destination_1 = value_1(match_value_2[1])
                        origin_1 = value_1(match_value_2[0])
                    
                    else: 
                        pattern_matching_result = 1
                    
                
                else: 
                    pattern_matching_result = 1
                
                if pattern_matching_result == 0:
                    return Frame(origin_1, destination_1, ms)
                
                elif pattern_matching_result == 1:
                    return None
                
            
            return choose(chooser, map_indexed(mapping, ani.m_sec, None), None)
        
    
    def arrow_427(ctx: Context=ctx, m: RawJny=m) -> Optional[FeatureCollection]:
        match_value_3 : Optional[RawAni] = m.ani
        if match_value_3 is None:
            return None
        
        else: 
            ani_1 : RawAni = match_value_3
            match_value_4 : Optional[RawPoly] = ani_1.poly
            if match_value_4 is None:
                match_value_5 : Optional[PolyG] = ani_1.poly_g
                if match_value_5 is None:
                    return None
                
                else: 
                    poly_g : PolyG = match_value_5
                    idx : int = poly_g.poly_xl[0] or 0
                    return ctx.common.polylines[idx] if (idx < len(ctx.common.polylines)) else None
                
            
            else: 
                value : RawPoly = match_value_4
                return Profile__get_parsePolyline(ctx.profile)(ctx)(value)
            
        
    
    return Movement(m.dir_txt, m.jid, get_element_at(m.prod_x, ctx.common.lines), arrow_425(), Profile__get_parseStopovers(ctx.profile)(ctx)(m.stop_l)(value_1(m.date)), arrow_426(), arrow_427())





from typing import (TypeVar, Callable, Optional, Tuple, Any)
from .big_int import (from_zero, op_addition)
from .char import char_code_at
from .decimal import (from_parts, op_addition as op_addition_1)
from .long import (from_bits, op_addition as op_addition_2)
from .seq import (delay, unfold)
from .util import (compare, IEnumerable)

_T = TypeVar("_T")

def make_range_step_function(step: _T, stop: _T, zero: _T, add: Callable[[_T, _T], _T]) -> Callable[[_T], Optional[Tuple[_T, _T]]]:
    step_compared_with_zero : int = compare(step, zero) or 0
    if step_compared_with_zero == 0:
        raise Exception("The step of a range cannot be zero")
    
    step_greater_than_zero : bool = step_compared_with_zero > 0
    def arrow_138(x: Optional[_T]=None, step: _T=step, stop: _T=stop, zero: _T=zero, add: Callable[[_T, _T], _T]=add) -> Optional[Tuple[_T, _T]]:
        compared_with_last : int = compare(x, stop) or 0
        return ((x, add(x, step))) if (True if ((compared_with_last <= 0) if step_greater_than_zero else False) else ((compared_with_last >= 0) if (not step_greater_than_zero) else False)) else None
    
    return arrow_138


def integral_range_step(start: _T, step: _T, stop: _T, zero: _T, add: Callable[[_T, _T], _T]) -> IEnumerable[_T]:
    step_fn : Callable[[_T], Optional[Tuple[_T, _T]]] = make_range_step_function(step, stop, zero, add)
    def arrow_140(start: _T=start, step: _T=step, stop: _T=stop, zero: _T=zero, add: Callable[[_T, _T], _T]=add) -> IEnumerable[_T]:
        return unfold(step_fn, start)
    
    return delay(arrow_140)


def range_big_int(start: Any, step: Any, stop: Any) -> IEnumerable[Any]:
    def arrow_141(x: Any, y: Any, start: Any=start, step: Any=step, stop: Any=stop) -> Any:
        return op_addition(x, y)
    
    return integral_range_step(start, step, stop, from_zero(), arrow_141)


def range_decimal(start: Any, step: Any, stop: Any) -> IEnumerable[Any]:
    def arrow_142(x: Any, y: Any, start: Any=start, step: Any=step, stop: Any=stop) -> Any:
        return op_addition_1(x, y)
    
    return integral_range_step(start, step, stop, from_parts(0, 0, 0, False, 0), arrow_142)


def range_double(start: float, step: float, stop: float) -> IEnumerable[float]:
    def arrow_143(x: float, y: float, start: float=start, step: float=step, stop: float=stop) -> float:
        return x + y
    
    return integral_range_step(start, step, stop, 0, arrow_143)


def range_int64(start: Any, step: Any, stop: Any) -> IEnumerable[Any]:
    def arrow_144(x: Any, y: Any, start: Any=start, step: Any=step, stop: Any=stop) -> Any:
        return op_addition_2(x, y)
    
    return integral_range_step(start, step, stop, from_bits(0, 0, False), arrow_144)


def range_uint64(start: Any, step: Any, stop: Any) -> IEnumerable[Any]:
    def arrow_145(x: Any, y: Any, start: Any=start, step: Any=step, stop: Any=stop) -> Any:
        return op_addition_2(x, y)
    
    return integral_range_step(start, step, stop, from_bits(0, 0, True), arrow_145)


def range_char(start: str, stop: str) -> IEnumerable[str]:
    int_stop : int = char_code_at(stop, 0) or 0
    def arrow_146(start: str=start, stop: str=stop) -> IEnumerable[str]:
        def step_fn(c: int) -> Optional[Tuple[str, int]]:
            if c <= int_stop:
                return (chr(c), c + 1)
            
            else: 
                return None
            
        
        return unfold(step_fn, char_code_at(start, 0))
    
    return delay(arrow_146)





from __future__ import annotations
from typing import (Optional, Tuple, Any)
from ...fable_library.double import parse
from ...fable_library.int32 import parse as parse_1
from ...fable_library.string import (to_console, printf, substring)
from ...fs_hafas_python.context import (Profile, Context)
from ...fs_hafas_python.extensions.date_time_offset_ex import (DateTimeOffsetEx__AddDays_5E38073B, parse_date_time_with_offset, DateTimeOffsetEx, DateTimeOffsetEx__get_DateTime, ToIsoString)

def parse_date_time_ex(profile: Profile, date: str, time: Optional[str]=None, tz_offset: Optional[int]=None) -> Tuple[Optional[Any], Optional[str]]:
    try: 
        if time is None:
            return (None, None)
        
        else: 
            time_1 : str = time
            pattern_input : Tuple[str, float] = ((substring(time_1, len(time_1) - 6, 6), parse(substring(time_1, 0, 2)))) if (len(time_1) > 6) else ((time_1, 0))
            time6 : str = pattern_input[0]
            dto : DateTimeOffsetEx = DateTimeOffsetEx__AddDays_5E38073B(parse_date_time_with_offset(profile, parse_1(substring(date, 0, 4), 511, False, 32), parse_1(substring(date, 4, 2), 511, False, 32), parse_1(substring(date, 6, 2), 511, False, 32), parse_1(substring(time6, 0, 2), 511, False, 32), parse_1(substring(time6, 2, 2), 511, False, 32), parse_1(substring(time6, 4, 2), 511, False, 32), tz_offset), pattern_input[1])
            return (DateTimeOffsetEx__get_DateTime(dto), ToIsoString(dto))
        
    
    except Exception as ex:
        to_console(printf("parseDateTimeEx: %A"))(ex)
        return (None, None)
    


def parse_date_time(ctx: Context, date: str, time: Optional[str]=None, tz_offset: Optional[int]=None) -> Optional[str]:
    return parse_date_time_ex(ctx.profile, date, time, tz_offset)[1]


